/**
 * Estudiant: David Carreras Gresely
 * TFC: Disseny i implementaci de la base de dades dun sistema 
 *      de gesti d'informaci de jugadors de bsquet.
 * Data: 12-06-2013
 * Document: Creacio_Proc_ABM.sql
 * Contingut: Aquest document cont els scripts per crear els Procediments de
 *            Alta, Baixa i Modificaci (ABM) de jugadors, Contractes i Partits.
 */
 
/**
 * Especificacions comunes a tots els Procediemnts:
 *
 *    Tots els procediments disposen de tractament dexcepcions per gestionar els errors mes probables. 
 *    En cas dexcepci, el procediment guarda el codi i el missatge derror en el parmetre de 
 *    sortida RSP. Si no hi ha hagut cap error el contingut de RSP ser OK. En el subapartat 
 *    Codis derror que trobarem mes avall en la descripci de cada un dels procediments, hi ha la 
 *    relaci dels errors que el procediment captura, els codis i missatges derror que el procediment 
 *    retorna, i la descripci de les causes que provoquen el llanament de lexcepci.
 *    A mes a mes, cada procediment guarda en la taula log un registre de la crida feta al procediment. 
 *    Per tal dassegurar que el registre es guardar fins i tot en cas derror, he implementat tots els 
 *    procediments creant a dins del bloc principal BEGIN...END, un sub-bloc on hi poso totes les 
 *    operacions que fa el procediment, excepte guardar el registre en la taula log. A mes a mes, el 
 *    sub-bloc captura les seves prpies excepcions. Daquesta forma, encara que es llenci una excepci 
 *    des del sub-bloc, lexecuci del procediment continuar en el bloc principal, i per tan, sempre es 
 *    guardar lerror en la taula log.
 *    
 *    Dades guardades en la taula log:
 *    ================================================================================================
 *    nom	          tipus	               descripci
 *    ================================================================================================
 *    data_hora	      TIMESTAMP            Especifica la data i lhora en que sha cridat al 
 *                                         procediment. Per defecte, tots els procediments li 
 *                                         assignen la data i lhora actual del sistema (SYSTIMESTAMP) 
 *    ------------------------------------------------------------------------------------------------
 *    procediment	  VARCHAR2(45 CHAR)    Es un string que especifica el nom del procediment cridat.
 *    ------------------------------------------------------------------------------------------------
 *    parametres_in	  VARCHAR2(2000 CHAR)  Es la llista de parmetres dentrada del procediment amb 
 *                                         els seus valors, representada com un llarg string
 *    ------------------------------------------------------------------------------------------------
 *    parametres_out  VARCHAR2(2000 CHAR)  Es la llista de parmetres de sortida del procediment 
 *                                         amb els seus valors, representada com un llarg string.
 *    ================================================================================================
 */








/**
 * -------------------------------------------------------------------------------
 * -- Procediment ALTA_JUGADOR 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet donar dalta a un nou jugador en la B.D. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                  tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * id_representant      NUMBER    IN	Enter que representa el Id del representant. Cal que el     Un enter que coincideixi amb 
 *                                      representant hagi estat prviament donat dalta a la BD.	 el Id del representant
 * -------------------------------------------------------------------------------------------------------------------------------
 * id_posicio_joc       NUMBER    IN	Enter que representa el Id de la posici de joc. Cal que    Un enter que coincideixi amb 
 *                                      la posici de joc hagi estat prviament donada dalta a      el Id de la posici de joc
 *                                      la BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * genere               NUMBER    IN	Enter que representa el Id del gnere de la competici.     1=mascul
 *                                      Cal que el gnere hagi estat prviament donat dalta a      2=femen
 *                                      la BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * modalitat	        NUMBER    IN	Enter que representa el Id de la modalitat de la            1=normal
 *                                      competici. Cal que la modalitat hagi estat prviament      2=cadira de rodes
 *                                      donada dalta a la BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * nacionalitat         VARCHAR2  IN	Especifica el pais del jugador. Es sensible a majscules    Un string amb el nom del pais 
 *                                      i minscules. I tamb es sensible als accents i altres       (max.20 carcters)
 *                                      signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * federacio            VARCHAR2  IN	Especifica el pais de on es la federaci. Es sensible a     Un string amb el nom del pais 
 *                                      majscules i minscules. I tamb es sensible als accents     (max.20 carcters)
 *                                      i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * num_federat          VARCHAR2  IN	Especifica el nmero de federat del jugador. Es sensible    String que pot contenir 
 *                                      a majscules i minscules.	                                 lletres i nmeros (max.20 
 *                                                                                                   carcters)
 * -------------------------------------------------------------------------------------------------------------------------------
 * nom                  VARCHAR2  IN	Especifica el nom del jugador. Es sensible a majscules     Un string (max.20 carcters)
 *                                      i minscules. I tamb es sensible als accents i altres 
 *                                      signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * cognoms	            VARCHAR2  IN	Especifica els cognoms del jugador. Es sensible a           Un string (max.45 carcters)
 *                                      majscules i minscules. I tamb es sensible als accents 
 *                                      i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * data_naixement       DATE      IN	Especifica la data de naixement del jugador. El format      Un string de 10 carcters 
 *                                      de la data te que ser dd-mm-aaaa o be dd/mm/aaaa	     representant la data
 * -------------------------------------------------------------------------------------------------------------------------------
 * altura               FLOAT     IN	Especifica lalada del jugador expressada en metres. 	    N amb decimals (p.ex.: 2.05)
 * -------------------------------------------------------------------------------------------------------------------------------
 * pes                  FLOAT     IN	Especifica el pes del jugador expressat en kilograms.	    N amb decimals (p.ex.: 93.5)
 * -------------------------------------------------------------------------------------------------------------------------------
 * url_web              VARCHAR2  IN	Especifica ladrea web del jugador. Els formats poden      NULL
 *                                      ser http://www.adrea.com o be www.adrea.com           String amb ladrea web
 *                                      (NOTA: en la taula jugador, ladrea no es guarda amb 
 *                                      el tipus VARCHAR2, sin amb el tipus HttpUriType)	
 * -------------------------------------------------------------------------------------------------------------------------------
 * url_video            VARCHAR2  IN	Especifica ladrea web de un video del jugador. Els        NULL
 *                                      formats poden ser http://www.adrea.com o be              String amb ladrea web
 *                                      www.adrea.com  (NOTA: en la taula jugador, ladrea 
 *                                      no es guarda amb el tipus VARCHAR2, sin amb el tipus 
 *                                      HttpUriType)	
 * -------------------------------------------------------------------------------------------------------------------------------
 * activitat            VARCHAR2  IN	Especifica si el jugador est en actiu o retirat. Es        'actiu'
 *                                      sensible a majscules i minscules. I tamb es sensible     'retirat'
 *                                      als accents i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * estat                VARCHAR2  IN	Especifica lestat del jugador. Es sensible a majscules    'alta'
 *                                      i minscules. I tamb es sensible als accents i altres      'baixa'
 *                                      signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * sub_estat            VARCHAR2  IN	Especifica si la baixa del jugador es per motius mdics     NULL
 *                                      o personals. Es sensible a majscules i minscules. I       baixa mdica
 *                                      tamb es sensible als accents i altres signes de            baixa per motius personals
 *                                      puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * diagnostic_actual    VARCHAR2  IN	Es una descripci del diagnstic mdic del jugador en       NULL
 *                                      cas de baixa mdica	                                        (max.2000 carcters)
 * -------------------------------------------------------------------------------------------------------------------------------
 * data_disponibilitat  DATE      IN	Especifica la data en que es preveu que el jugador          NULL
 *                                      tornar a tenir lalta mdica. El format de la data te      Un string de 10 carcters 
 *                                      que ser dd-mm-aaaa o be dd/mm/aaaa	                     representant la data
 * ===============================================================================================================================
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP	VARCHAR2    OUT	    Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OKsi tot ha  
 *                          lutilitzar el procediment per passar al software            anat be.
 *                          client informaci sobre si hi ha hagut una excepci o no.    Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.  
 * ===============================================================================================================================
 * 
 * 
 *
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                  Quan es llena
 * ===============================================================================================================================
 * -20000	     PARAMETRE_NUL	    ORA-20000: Hi ha un o mes parametres nuls.	  Excepci definida per el procediment, que es 
 *                                                                                llena quan un o mes parmetres dentrada (noms 
 *                                                                                els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2290	     CHECK_ERROR	    ORA-2290: check constraint (DAVID.[nom del    Excepci definida internament i anomenada  per 
 *                                  CHECK CONSTRAINT]) violated. El valor del     el procediment, que es llena quan un parmetre 
 *                                  parmetre `[nom del parmetre] noms pot     dentrada te un valor que no coincideix amb els 
 *                                  ser `[valors].	                              valors permesos per el CHECK. En el missatge 
 *                                                                                derror sindica el nom de la restricci, el nom 
 *                                                                                del parmetre i els valors que pot tenir.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291	     FK_ERRONIA	        ORA-2291: integrity constraint ([nom del      Excepci definida internament i anomenada  per 
 *                                  constraint]) violated- parent key not found.  el procediment, que es llena quan un dels 
 *                                                                                parmetres dentrada fa referncia a una clau 
 *                                                                                forana amb un valor que no hi es. En el missatge 
 *                                                                                derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 * 		         OTHERS                                                           Amb aquest nom dexcepci, el procediment 
 *                                                                                captura qualsevol altre excepci no especificada 
 *                                                                                abans.  El codi derror i el missatge dependran 
 *                                                                                de la causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE alta_jugador
( RSP                 OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, id_representant     IN  NUMBER
, id_posicio_joc      IN  NUMBER
, genere              IN  NUMBER
, modalitat           IN  NUMBER
, nacionalitat        IN  VARCHAR2
, federacio           IN  VARCHAR2
, num_federat         IN  VARCHAR2
, nom                 IN  VARCHAR2
, cognoms             IN  VARCHAR2
, data_naixement      IN  DATE
, altura              IN  FLOAT
, pes                 IN  FLOAT
, url_web             IN  VARCHAR2 := NULL
, url_video           IN  VARCHAR2 := NULL
, activitat           IN  VARCHAR2
, estat               IN  VARCHAR2
, sub_estat           IN  VARCHAR2 := NULL
, diagnostic_actual   IN  VARCHAR2 := NULL
, data_disponibilitat IN  DATE     := NULL ) IS 

  parametre_nul EXCEPTION;                              -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);        -- assigna un codi de error a la excepcio
  check_error EXCEPTION;                                -- declara una excepcio
  PRAGMA EXCEPTION_INIT (check_error, -2290);           -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                                 -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);            -- assigna un codi de error a la excepci
  id_jugador NUMBER;                                    -- hi guardarem la seguent PK de la taula jugador
  id_log NUMBER;                                        -- hi guardarem la seguent PK de la taula log
  p_url_web HttpUriType;                                -- sutilitza per canviar el tipus de la URL
  p_url_video HttpUriType;                              -- sutilitza per canviar el tipus de la URL
  errmsg VARCHAR2(255);                                 -- sutilitza per ampliar els missatges derror

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT alta_jugador;
    
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF id_representant IS NOT NULL AND 
       id_posicio_joc IS NOT NULL  AND 
       genere IS NOT NULL          AND
       modalitat IS NOT NULL       AND
       nacionalitat IS NOT NULL    AND 
       federacio IS NOT NULL       AND 
       num_federat IS NOT NULL     AND 
       nom IS NOT NULL             AND 
       cognoms IS NOT NULL         AND 
       data_naixement IS NOT NULL  AND 
       altura IS NOT NULL          AND 
       pes IS NOT NULL             AND 
       activitat IS NOT NULL       AND 
       estat IS NOT NULL           THEN
  
       -- Comprova que el valor de 'activitat' sigui correcte, i amplia el missatge en cas derror.
       IF activitat NOT IN ('actiu', 'retirat') THEN 
         errmsg := 'El valor del parmetre `activitat noms pot ser `actiu o `retirat.';
       END IF;
  
       -- Comprova que el valor de 'estat' sigui correcte, i amplia el missatge en cas derror.
       IF estat NOT IN ('alta','baixa') THEN 
         errmsg := 'El valor del parmetre `estat noms pot ser `alta o `baixa.';
       END IF;
  
       -- Comprova que el valor de 'sub_estat' sigui correcte, i amplia el missatge en cas derror.
       IF sub_estat NOT IN ('alta', 'baixa medica', 'baixa per motius personals') THEN 
         errmsg := 'El valor del parmetre `sub_estat noms pot ser `alta, `baixa medica o `baixa per motius personals.';
       END IF;
  
       -- Consulta la sequencia que indica el seguent valor de la PK de la taula jugador.
       id_jugador := jugador_seq.nextval;
       
       -- converteix el format de les adreces web
       p_url_web := httpuritype.createUri(url_web);
       p_url_video := httpuritype.createUri(url_video);
       
       -- Guarda les dades del nou jugador a la taula 'jugador'.
       INSERT INTO jugador VALUES
       ( id_jugador
       , id_representant
       , id_posicio_joc
       , genere
       , modalitat
       , nacionalitat
       , federacio
       , num_federat
       , nom
       , cognoms
       , data_naixement
       , altura
       , pes
       , p_url_web
       , p_url_video
       , activitat
       , estat
       , sub_estat
       , diagnostic_actual
       , data_disponibilitat);
    ELSE
       -- Si un dels parametres es nul, llena una excepci
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parmetre de sortida indicar OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO alta_jugador;                  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parmetre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN check_error THEN                        -- En cas que un parmetre faci un error de CHECK,
        ROLLBACK TO alta_jugador;                  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2290, '||SQLERRM||'. '||errmsg;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO alta_jugador;                  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO alta_jugador;                  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parmetre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
    
-- PASSA A GUARDAR LES DADES EN LA TAULA log
  
  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                   -- la data i lhora en que sha cridat al procediment
  , 'alta_jugador'                                 -- el nom del procediment cridat
  ,      'id_representant = ' || id_representant   -- la llista de parmetres dentrada (com un llarg string)
     || ', id_posicio_joc = ' || id_posicio_joc
     || ', genere = '         || genere
     || ', modalitat = '      || modalitat
     || ', nacionalitat = '   || nacionalitat
     || ', federacio = '      || federacio
     || ', num_federat = '    || num_federat
     || ', nom = '            || nom
     || ', cognoms = '        || cognoms
     || ', data_naixement = ' || data_naixement
     || ', altura = '         || altura
     || ', pes = '            || pes
     || ', url_web = '        || url_web
     || ', url_video = '      || url_video
     || ', activitat = '      || activitat
     || ', estat = '          || estat
     || ', sub_estat = '      || sub_estat
     || ', diagnostic_actual = '   || diagnostic_actual
     || ', data_disponibilitat = ' || data_disponibilitat
  , 'RSP = ' || RSP);                                        -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END alta_jugador; 
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment BAIXA_JUGADOR 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet donar de baixa a un nou jugador en la B.D. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                  tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_id_jugador	        NUMBER    IN    Enter que representa el Id del jugador. Cal que el          Un enter que coincideixi amb 
 *                                      jugador hagi estat prviament donat dalta a la BD.	         el Id del jugador
 * ===============================================================================================================================
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP	VARCHAR2    OUT     Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha  
 *                          lutilitzar el procediment per passar al software            anat be.
 *                          client informaci sobre si hi ha hagut una excepci o no.	 Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.
 * ===============================================================================================================================
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci  Missatge derror retornat	                Quan es llena
 * ===============================================================================================================================
 * -20000	     PARAMETRE_NUL	    ORA-20000: Hi ha un o mes parametres nuls.	Excepci definida per el procediment, que es 
 *                                                                              llena quan un o mes parmetres dentrada son NULL, 
 *                                                                              o un string buit, o no hi son
 * -------------------------------------------------------------------------------------------------------------------------------
 * 		         OTHERS                                                         Amb aquest nom dexcepci, el procediment captura 
 *                                                                              qualsevol altre excepci no especificada abans.  
 *                                                                              El codi derror i el missatge dependran de la 
 *                                                                              causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE procedure baixa_jugador
( RSP          OUT VARCHAR2                           -- retornara el codi derror si nhi ha hagut.
, p_id_jugador IN  NUMBER ) IS 

  parametre_nul EXCEPTION;                            -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);      -- assigna un codi de error a la excepcio
  id_log NUMBER;                                      -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT baixa_jugador;
    
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_id_jugador IS NOT NULL THEN
  
       -- Esborra el jugador de la taula 'jugador'.
       DELETE FROM jugador WHERE id_jugador = p_id_jugador;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO baixa_jugador;                 -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO baixa_jugador;                 -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
    
  -- PASSA A GUARDAR LES DADES EN LA TAULA log
  
-- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'baixa_jugador'                            -- el nom del procediment cridat
  , 'id_jugador = ' || p_id_jugador            -- la llista de parmetres dentrada (com un llarg string)
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END baixa_jugador;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment MODIFICA_JUGADOR 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet modificar les dades dun jugador que ja existeix en la B.D. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                   tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_id_jugador          NUMBER    IN   Enter que representa el Id del jugador. Cal que el          Un enter que coincideixi amb 
 *                                      jugador hagi estat prviament donat dalta a la BD.	         el Id del jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_id_representant     NUMBER    IN   Enter que representa el Id del representant. Cal que el     Un enter que coincideixi amb 
 *                                      representant hagi estat prviament donat dalta a la BD.	 el Id del representant
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_id_posicio_joc      NUMBER    IN   Enter que representa el Id de la posici de joc. Cal que    Un enter que coincideixi amb 
 *                                      la posici de joc hagi estat prviament donada dalta a      el Id de la posici de joc
 *                                      la BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_genere              NUMBER    IN   Enter que representa el Id del gnere de la competici.     1= mascul
 *                                      Cal que el gnere hagi estat prviament donat dalta a      2= femen
 *                                      la BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_modalitat	         NUMBER    IN   Enter que representa el Id de la modalitat de la            1= normal
 *                                      competici. Cal que la modalitat hagi estat prviament      2= cadira de rodes
 *                                      donada dalta a la BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_nacionalitat        VARCHAR2  IN   Especifica el pais del jugador. Es sensible a majscules    Un string amb el nom del pais 
 *                                      i minscules. I tamb es sensible als accents i altres       (max.20 carcters)
 *                                      signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_federacio           VARCHAR2  IN   Especifica el pais de on es la federaci. Es sensible a     Un string amb el nom del pais 
 *                                      majscules i minscules. I tamb es sensible als accents     (max.20 carcters)
 *                                      i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_num_federat         VARCHAR2  IN   Especifica el nmero de federat del jugador. Es sensible    String que pot contenir 
 *                                      a majscules i minscules.	                                 lletres i nmeros (max.20 
 *                                                                                                   carcters)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_nom                 VARCHAR2  IN   Especifica el nom del jugador. Es sensible a majscules     Un string (max.20 carcters)
 *                                      i minscules. I tamb es sensible als accents i altres 
 *                                      signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_cognoms             VARCHAR2  IN   Especifica els cognoms del jugador. Es sensible a           Un string (max.45 carcters)
 *                                      majscules i minscules. I tamb es sensible als accents 
 *                                      i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_data_naixement      DATE      IN   Especifica la data de naixement del jugador. El format      Un string de 10 carcters 
 *                                      de la data te que ser dd-mm-aaaa o be dd/mm/aaaa	     representant la data
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_altura              FLOAT     IN   Especifica lalada del jugador expressada en metres.       N amb decimals (p.ex.: 2.05)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_pes                 FLOAT     IN   Especifica el pes del jugador expressat en kilograms.       N amb decimals (p.ex.: 93.5)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_url_web             VARCHAR2  IN   Especifica ladrea web del jugador. Els formats poden      NULL
 *                                      ser http://www.adrea.com o be www.adrea.com           String amb ladrea web
 *                                      (NOTA: en la taula jugador, ladrea no es guarda amb 
 *                                      el tipus VARCHAR2, sin amb el tipus HttpUriType)	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_url_video           VARCHAR2  IN   Especifica ladrea web de un video del jugador. Els        NULL
 *                                      formats poden ser http://www.adrea.com o be              String amb ladrea web
 *                                      www.adrea.com  (NOTA: en la taula jugador, ladrea 
 *                                      no es guarda amb el tipus VARCHAR2, sin amb el tipus 
 *                                      HttpUriType)	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_activitat           VARCHAR2  IN   Especifica si el jugador est en actiu o retirat. Es        'actiu'
 *                                      sensible a majscules i minscules. I tamb es sensible     'retirat'
 *                                      als accents i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_estat               VARCHAR2  IN   Especifica lestat del jugador. Es sensible a majscules    'alta'
 *                                      i minscules. I tamb es sensible als accents i altres      'baixa'
 *                                      signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_sub_estat           VARCHAR2  IN   Especifica si la baixa del jugador es per motius mdics    NULL
 *                                      o personals. Es sensible a majscules i minscules.        baixa mdica
 *                                      I tamb es sensible als accents i altres signes de         baixa per motius personals
 *                                      puntuaci.
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_diagnostic_actual	  VARCHAR2 IN   Es una descripci del diagnstic mdic del jugador en      NULL
 *                                      cas de baixa mdica                                        (max.2000 carcters)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_data_disponibilitat  DATE     IN   Especifica la data en que es preveu que el jugador         NULL
 *                                      tornar a tenir lalta mdica. El format de la data        Un string de 10 carcters 
 *                                      te que ser dd-mm-aaaa o be dd/mm/aaaa	                representant la data
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP	VARCHAR2   OUT      Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software            anat be.
 *                          client informaci sobre si hi ha hagut una excepci o no.	 Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci. 
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                  Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parmetres nuls.    Excepci definida per el procediment, que es 
 *                                                                                llena quan un o mes parmetres dentrada son 
 *                                                                                NULL, o un string buit, o no hi son
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2290         CHECK_ERROR        ORA-2290: check constraint (DAVID.[nom del    Excepci definida internament i anomenada  per 
 *                                  CHECK CONSTRAINT]) violated. El valor del     el procediment, que es llena quan un parmetre 
 *                                  parmetre `[nom del parmetre] noms pot     dentrada te un valor que no coincideix amb els 
 *                                  ser `[valors].	                              valors permesos per el CHECK. En el missatge 
 *                                                                                derror sindica el nom de la restricci, el nom 
 *                                                                                del parmetre i els valors que pot tenir.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del      Excepci definida internament i anomenada per el 
 *                                  constraint]) violated - parent key not        procediment, que es llena quan un dels 
 *                                  found.	                                      parmetres dentrada fa referncia a una clau 
 *                                                                                forana amb un valor que no hi es. En el missatge 
 *                                                                                derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                           Amb aquest nom dexcepci, el procediment 
 *                                                                                captura qualsevol altre excepci no especificada 
 *                                                                                abans.  El codi derror i el missatge dependran 
 *                                                                                de la causa del error.
 * ===============================================================================================================================
 * 
 */ 


CREATE OR REPLACE 
PROCEDURE modifica_jugador
( RSP                   OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, p_id_jugador          IN  NUMBER
, p_id_representant     IN  NUMBER
, p_id_posicio_joc      IN  NUMBER
, p_genere              IN  NUMBER
, p_modalitat           IN  NUMBER
, p_nacionalitat        IN  VARCHAR2
, p_federacio           IN  VARCHAR2
, p_num_federat         IN  VARCHAR2
, p_nom                 IN  VARCHAR2
, p_cognoms             IN  VARCHAR2
, p_data_naixement      IN  DATE
, p_altura              IN  FLOAT
, p_pes                 IN  FLOAT
, p_url_web             IN  VARCHAR2
, p_url_video           IN  VARCHAR2
, p_activitat           IN  VARCHAR2
, p_estat               IN  VARCHAR2
, p_sub_estat           IN  VARCHAR2
, p_diagnostic_actual   IN  VARCHAR2
, p_data_disponibilitat IN  DATE ) IS 

  parametre_nul EXCEPTION;                            -- declara una excepci
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);      -- assigna un codi de error a la excepci
  check_error EXCEPTION;                              -- declara una excepci
  PRAGMA EXCEPTION_INIT (check_error, -2290);         -- assigna un codi de error a la excepci
  fk_erronia EXCEPTION;                               -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);          -- assigna un codi de error a la excepci
  id_log NUMBER;                                      -- hi guardarem la seguent PK de la taula log
  p_url_web2 HttpUriType;                             -- sutilitza per canviar el tipus de la URL
  p_url_video2 HttpUriType;                           -- sutilitza per canviar el tipus de la URL
  errmsg VARCHAR2(255);                               -- sutilitza per ampliar els missatges derror

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT modifica_jugador;
    
    -- Comprova que les claus foranes no tinguin un valor nul.
    IF p_id_representant IS NOT NULL AND 
       p_id_posicio_joc IS NOT NULL  AND
       p_genere IS NOT NULL          AND
       p_modalitat IS NOT NULL       THEN
  
       -- Comprova que el valor de 'activitat' sigui correcte, i amplia el missatge en cas derror.
       IF activitat NOT IN ('actiu', 'retirat') THEN 
         errmsg := 'El valor del parmetre `activitat noms pot ser `actiu o `retirat.';
       END IF;
  
       -- Comprova que el valor de 'estat' sigui correcte, i amplia el missatge en cas derror.
       IF estat NOT IN ('alta','baixa') THEN 
         errmsg := 'El valor del parmetre `estat noms pot ser `alta o `baixa.';
       END IF;
  
       -- Comprova que el valor de 'sub_estat' sigui correcte, i amplia el missatge en cas derror.
       IF sub_estat NOT IN ('alta', 'baixa medica', 'baixa per motius personals') THEN 
         errmsg := 'El valor del parmetre `sub_estat noms pot ser `alta, `baixa medica o `baixa per motius personals.';
       END IF;
       
       -- converteix el format de les adreces web
       p_url_web2 := httpuritype.createUri(p_url_web);
       p_url_video2 := httpuritype.createUri(p_url_video);
  
       -- Guarda els canvis del jugador a la taula 'jugador'.
       UPDATE jugador SET 
         representant = p_id_representant
       , posicio_joc = p_id_posicio_joc
       , genere = p_genere
       , modalitat = p_modalitat
       , nacionalitat = p_nacionalitat
       , federacio = p_federacio
       , num_federat = p_num_federat
       , nom = p_nom
       , cognoms = p_cognoms
       , data_naixement = p_data_naixement
       , altura = p_altura
       , pes = p_pes
       , url_web = p_url_web2
       , url_video = p_url_video2
       , activitat = p_activitat
       , estat = p_estat
       , sub_estat = p_sub_estat
       , diagnostic_actual = p_diagnostic_actual
       , data_disponibilitat = p_data_disponibilitat 
       WHERE id_jugador = p_id_jugador;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parmetres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN clau_forana_nula THEN                   -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO modifica_jugador;              -- Ho desf tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN check_error THEN                        -- En cas que un parmetre faci un error de CHECK,
        ROLLBACK TO modifica _jugador;             -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2290, '||SQLERRM||'. '||errmsg;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO modifica_jugador;              -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO modifica_jugador;              -- Ho desf tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
    
-- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                  -- la data i lhora en que sha cridat al procediment
  , 'modifica_jugador'                            -- el nom del procediment cridat
  ,     'id_jugador = '        || p_id_jugador    -- la llista de parmetres dentrada (com un llarg string)
     || ', id_representant = ' || p_id_representant
     || ', id_posicio_joc = '  || p_id_posicio_joc
     || ', genere = '          || p_genere
     || ', modalitat = '       || p_modalitat
     || ', nacionalitat = '    || p_nacionalitat
     || ', federacio = '       || p_federacio
     || ', num_federat = '     || p_num_federat
     || ', nom = '             || p_nom
     || ', cognoms = '         || p_cognoms
     || ', data_naixement = '  || p_data_naixement
     || ', altura = '          || p_altura
     || ', pes = '             || p_pes
     || ', url_web = '         || p_url_web 
     || ', url_video = '       || p_url_video
     || ', activitat = '       || p_activitat
     || ', estat = '           || p_estat
     || ', sub_estat = '       || p_sub_estat
     || ', diagnostic_actual = '   || p_diagnostic_actual
     || ', data_disponibilitat = ' || p_data_disponibilitat
  , 'RSP = ' || RSP);                                        -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END modifica_jugador;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment ALTA_CONTRACTE 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet donar dalta un contracte en la B.D.  El mateix procediment serveix tan per una transacci de compravenda 
 *             dun jugador entre dos equips, com tamb per la contractaci dun jugador que en aquest moment no tingui cap equip 
 *             (deixant el camp equip_venedor amb un valor NULL). 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                    tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * jugador                NUMBER    IN    Enter que representa el Id del jugador. Cal que el jugador    Un enter que coincideixi 
 *                                        hagi estat prviament donat dalta a la BD.	                 amb el Id del jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * equip_comprador        NUMBER    IN    Enter que representa el Id de lequip comprador. Cal que      Un enter que coincideixi 
 *                                        lequip comprador hagi estat prviament donada dalta a        amb el Id de lequip 
 *                                        la BD.	                                                     comprador
 * -------------------------------------------------------------------------------------------------------------------------------
 * equip_venedor          NUMBER    IN    Enter que representa el Id de lequip venedor. Cal que        Un enter que coincideixi 
 *                                        lequip venedor hagi estat prviament donada dalta a          amb el Id de lequip 
 *                                        la BD.	                                                     venedor
 * -------------------------------------------------------------------------------------------------------------------------------
 * sou_brut_anual         FLOAT     IN    Especifica el sou brut anual del jugador, en euros            Un nombre amb fins a 3 
 *                                                                                                       decimals (p.ex. 120000.00)
 * -------------------------------------------------------------------------------------------------------------------------------
 * compensacio_economica  FLOAT     IN    Especifica la compensaci econmica a lequip venedor         Un nombre amb fins a 3 
 *                                                                                                       decimals
 * -------------------------------------------------------------------------------------------------------------------------------
 * data_signatura         DATE      IN    Especifica la data en que es signa el contracte. El format    Un string de 10 carcters 
 *                                        de la data te que ser dd-mm-aaaa o be dd/mm/aaaa	         representant la data
 * -------------------------------------------------------------------------------------------------------------------------------
 * data_finalitzacio      DATE      IN    Especifica la data en que finalitza el contracte. El format   NULL
 *                                        de la data te que ser dd-mm-aaaa o be dd/mm/aaaa. Si es   Un string de 10 carcters 
 *                                        posa un valor NULL, i sha especificat un valor a              representant la data
 *                                        durada_contracte, aleshores el procediment calcular 
 *                                        automticament el valor de data_finalitzacio	
 * -------------------------------------------------------------------------------------------------------------------------------
 * durada_contracte       FLOAT     IN    Especifica la duraci del contracte expressada en anys.       NULL
 *                                        Accepta tan anys enters (p.ex.: 3 anys), com tamb            Un enter indicant el 
 *                                        fraccions dany (p.ex.: 2.25 equivaldria a 2 anys i 4          nombre danys
 *                                        mesos). Si es posa un valor NULL, i sha especificat una      Un nmero amb decimals, 
 *                                        data adata_finalitzacio, aleshores el procediment            on la part entera son els 
 *                                        calcular automticament el valor de durada_contracte	     anys, i la part decimal 
 *                                                                                                       es una fracci dany.
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2    OUT     Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                  Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.    Excepci definida per el procediment, que es 
 *                                                                                llena quan un o mes parmetres dentrada (noms 
 *                                                                                els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * -20002        INTERVAL_ERRONI    ORA-20002: La duraci del contracte no        Excepci definida per el procediment, que es 
 *                                  coincideix amb l`interval de dates.	          llena quan la duraci indicada del contracte no 
 *                                                                                coincideix amb el interval de temps entre les 
 *                                                                                dates indicades de signatura i finalitzaci del 
 *                                                                                contracte.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del      Excepci definida internament i anomenada  per 
 *                                  constraint]) violated - parent key not        el procediment, que es llena quan un dels 
 *                                  found.                                        parmetres dentrada fa referncia a una clau 
 *                                                                                forana amb un valor que no hi es. En el missatge 
 *                                                                                derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                           Amb aquest nom dexcepci, el procediment 
 *                                                                                captura qualsevol altre excepci no especificada 
 *                                                                                abans.  El codi derror i el missatge dependran 
 *                                                                                de la causa del error.
 * ===============================================================================================================================
 * 
 */



CREATE OR REPLACE 
PROCEDURE alta_contracte
( RSP                   OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, jugador               IN  NUMBER
, equip_comprador       IN  NUMBER
, equip_venedor         IN  NUMBER  := NULL
, sou_brut_anual        IN  FLOAT   := 0
, compensacio_economica IN  FLOAT   := 0
, data_signatura        IN  DATE    := SYSDATE
, data_finalitzacio     IN  DATE    := NULL
, durada_contracte      IN  FLOAT   := NULL ) IS 

  parametre_nul EXCEPTION;                             -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);       -- assigna un codi de error a la excepcio
  interval_erroni EXCEPTION;                           -- declara una excepcio
  PRAGMA EXCEPTION_INIT (interval_erroni, -20002);     -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                                -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);           -- assigna un codi de error a la excepci
  id_contracte NUMBER;                                 -- hi guardarem la seguent PK de la taula jugador
  id_log NUMBER;                                       -- hi guardarem la seguent PK de la taula log
  valor_operacio FLOAT;                                -- hi guardarem el Calcul del valor de l'operacio
  data_fi DATE;        -- Guardara el calcul de la data final en cas que 'data_finalitzacio' sigui null
  duracio FLOAT;       -- Guardara el calcul de la durada en cas que 'durada_contracte' sigui null

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT alta_contracte;
    
    -- Comprova que no hi hagi cap parametre duna clau forana que sigui null, o que no en falti cap.
    IF jugador IS NOT NULL          AND 
       equip_comprador IS NOT NULL  AND
       data_signatura IS NOT NULL   THEN
  
       -- Decideix que fer segons si 'data_finalitzacio' i/o durada_contracte son null
       IF data_finalitzacio IS NULL AND
          durada_contracte IS NULL  THEN
             -- si els dos parametres son nuls es deixen tots dos nuls 
             -- (seria com un contracte fixa, de duraci indefinida)
             data_fi := NULL;
             duracio := NULL;
       ELSIF data_finalitzacio IS NULL THEN
          -- calcula la data de finalitzacio del contracte
          data_fi := data_signatura + NUMTOYMINTERVAL(durada_contracte,'year');
          duracio := durada_contracte;
       ELSIF durada_contracte IS NULL THEN
          -- calcula la duracio del contracte en ANYS (les fraccions d'any es guarden com a decimals)
          duracio := MONTHS_BETWEEN(data_finalitzacio , data_signatura)/12;
          data_fi := data_finalitzacio;
       ELSIF data_finalitzacio IS NOT NULL AND
             durada_contracte IS NOT NULL  THEN
             -- calcula l'interval entre les dates d'inici i final
             duracio := ROUND(MONTHS_BETWEEN(data_finalitzacio , data_signatura)/12 , 5);
             data_fi := data_finalitzacio;
             -- comprova que el parametre 'durada_contracte' coincideix amb l'interval calculat
             IF duracio != durada_contracte THEN 
                RAISE_APPLICATION_ERROR(-20002, 'La duracio del contracte no coincideix amb l`interval de dates.'); 
             END IF;
       END IF;
       
       -- Calcula el valor de l'operacio.
       valor_operacio := compensacio_economica + sou_brut_anual * duracio;
       
       -- Guarda les dades del nou contracte a la taula contracte.
       INSERT INTO contracte VALUES
       ( jugador
       , equip_comprador
       , equip_venedor
       , data_signatura
       , data_fi
       , duracio
       , sou_brut_anual
       , compensacio_economica
       , valor_operacio);
    ELSE
       -- Si una de les claus foranes es nulla, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parmetres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO alta_contracte;                -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN interval_erroni THEN       -- En cas que la duraci no coincideixi amb el interval entre dates
        ROLLBACK TO alta_contracte;                -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20002, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO alta_contracte;                -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO alta_contracte;                -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
    
-- PASSA A GUARDAR LES DADES EN LA TAULA log

-- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                   -- la data i lhora en que sha cridat al procediment
  , 'alta_contracte'                               -- el nom del procediment cridat
  ,       'jugador = '               || jugador    -- la llista de parmetres dentrada (com un llarg string)
     || ', equip_comprador = '       || equip_comprador
     || ', equip_venedor = '         || equip_venedor
     || ', sou_brut_anual = '        || sou_brut_anual
     || ', compensacio_economica = ' || compensacio_economica
     || ', data_signatura = '        || data_signatura
     || ', data_finalitzacio = '     || data_finalitzacio
     || ', durada_contracte = '      || durada_contracte
  , 'RSP = ' || RSP);                                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END alta_contracte;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment BAIXA_CONTRACTE 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet donar de baixa un contracte en la B.D.  
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_jugador          NUMBER   IN     Enter que representa el Id del jugador. Cal que el jugador    Un enter que coincideixi amb 
 *                                    hagi estat prviament donat dalta a la BD.	                 el Id del jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_equip_comprador  NUMBER   IN     Enter que representa el Id de lequip comprador. Cal que      Un enter que coincideixi amb 
 *                                    lequip comprador hagi estat prviament donada dalta a la     el Id de lequip comprador
 *                                    BD.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_data_signatura   DATE     IN     Especifica la data en que es signa el contracte. El format    Un string de 10 carcters 
 *                                    de la data te que ser dd-mm-aaaa o be dd/mm/aaaa	         representant la data
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2   OUT      Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                  Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.	  Excepci definida per el procediment, que es 
 *                                                                                llena quan un o mes parmetres dentrada que 
 *                                                                                corresponen a claus foranes son NULL, o un 
 *                                                                                string buit, o no hi son
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                           Amb aquest nom dexcepci, el procediment 
 *                                                                                captura qualsevol altre excepci no especificada 
 *                                                                                abans.  El codi derror i el missatge dependran 
 *                                                                                de la causa del error.
 * ===============================================================================================================================
 * 
 */



CREATE OR REPLACE 
PROCEDURE baixa_contracte
( RSP               OUT VARCHAR2                        -- retornara el codi derror si nhi ha hagut.
, p_jugador         IN  NUMBER
, p_equip_comprador IN  NUMBER
, p_data_signatura  IN  DATE ) IS 

  parametre_nul EXCEPTION;                              -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);        -- assigna un codi de error a la excepcio
  id_log NUMBER;                                        -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT baixa_contracte;
    
    -- Comprova que les claus foranes no tinguin un valor nul.
    IF p_jugador IS NOT NULL         AND 
       p_equip_comprador IS NOT NULL AND
       p_data_signatura IS NOT NULL  THEN
  
       -- Esborra el contracte de la taula 'contracte'.
       DELETE FROM contracte 
       WHERE jugador = p_jugador 
         AND equip_comprador = p_equip_comprador 
         AND TRUNC(data_signatura) = TRUNC(p_data_signatura);
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO baixa_contracte;               -- Ho desf tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO baixa_contracte;               -- Ho desf tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  END;

  -- PASSA A GUARDAR LES DADES EN LA TAULA log
  
-- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                  -- la data i lhora en que sha cridat al procediment
  , 'baixa_contracte'                             -- el nom del procediment cridat
  ,     ', jugador = '         || p_jugador       -- la llista de parmetres dentrada (com un llarg string)
     || ', equip_comprador = ' || p_equip_comprador
     || ', data_signatura = '  || p_data_signatura
  , 'RSP = ' || RSP);                             -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END baixa_contracte;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment MODIFICA_CONTRACTE 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet modificar les dades dun contracte que ja existeix en la B.D. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                      tipus  IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_jugador                NUMBER   IN    Enter que representa el Id del jugador. Cal que el           Un enter que coincideixi 
 *                                         jugador hagi estat prviament donat dalta a la BD.	         amb el Id del jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_equip_comprador        NUMBER   IN    Enter que representa el Id de lequip comprador. Cal que     Un enter que coincideixi 
 *                                         lequip comprador hagi estat prviament donada dalta a       amb el Id de lequip 
 *                                         la BD.	                                                     comprador
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_equip_venedor          NUMBER   IN    Enter que representa el Id de lequip venedor. Cal que       Un enter que coincideixi 
 *                                         lequip venedor hagi estat prviament donada dalta a         amb el Id de lequip 
 *                                         la BD.	                                                     venedor
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_sou_brut_anual         FLOAT    IN    Especifica el sou brut anual del jugador, en euros           Un nombre amb fins a 3 
 *                                                                                                       decimals (p.ex. 120000.00)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_compensacio_economica  FLOAT    IN    Especifica la compensaci econmica a lequip venedor        Un nombre amb fins a 3 
 *                                                                                                       decimals
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_data_signatura         DATE     IN    Especifica la data en que es signa el contracte. El format   Un string de 10 carcters 
 *                                         de la data te que ser dd-mm-aaaa o be dd/mm/aaaa	         representant la data
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_data_finalitzacio      DATE     IN    Especifica la data en que finalitza el contracte. El         NULL
 *                                         format de la data te que ser dd-mm-aaaa o be               Un string de 10 carcters 
 *                                         dd/mm/aaaa. Si esposa un valor NULL, i sha especificat     representant la data
 *                                         un valor a p_durada_contracte, aleshores el procediment 
 *                                         calcular automticament el valor de p_data_finalitzacio	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_durada_contracte       FLOAT    IN    Especifica la duraci del contracte expressada en anys.      NULL
 *                                         Accepta tan anys enters (p.ex.: 3 anys), com tamb           Un enter indicant el 
 *                                         fraccions dany (p.ex.: 2.25 equivaldria a 2 anys i 4         nombre danys
 *                                         mesos). Si es posa un valor NULL, i sha especificat         Un nmero amb decimals, 
 *                                         una data a p_data_finalitzacio, aleshores el procediment    on la part entera son els 
 *                                         calcular automticament el valor de p_durada_contracte     anys, i la part decimal 
 *                                                                                                       es una fracci dany.
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2   OUT      Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.  
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                  Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parmetres nuls.    Excepci definida per el procediment, que es 
 *                                                                                llena quan un o mes parmetres dentrada (noms 
 *                                                                                els IN) es NULL, o un string buit, o no hi es.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -20002        INTERVAL_ERRONI    ORA-20002: La duraci del contracte no        Excepci definida per el procediment, que es 
 *                                  coincideix amb l`interval de dates.	          llena quan la duraci indicada del contracte no 
 *                                                                                coincideix amb el interval de temps entre les 
 *                                                                                dates indicades de signatura i finalitzaci del 
 *                                                                                contracte.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del      Excepci definida internament i anomenada  per 
 *                                  constraint]) violated - parent key not found. el procediment, que es llena quan un dels 
 *                                                                                parmetres dentrada fa referncia a una clau 
 *                                                                                forana amb un valor que no hi es. En el missatge 
 *                                                                                derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                           Amb aquest nom dexcepci, el procediment 
 *                                                                                captura qualsevol altre excepci no especificada 
 *                                                                                abans.  El codi derror i el missatge dependran 
 *                                                                                de la causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE 
PROCEDURE modifica_contracte
( RSP                     OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, p_jugador               IN  NUMBER
, p_equip_comprador       IN  NUMBER
, p_equip_venedor         IN  NUMBER  := NULL
, p_sou_brut_anual        IN  NUMBER  := 0
, p_compensacio_economica IN  NUMBER  := 0
, p_data_signatura        IN  DATE    := SYSDATE
, p_data_finalitzacio     IN  DATE    := NULL
, p_durada_contracte      IN  NUMBER  := NULL ) IS 

  parametre_nul EXCEPTION;                              -- declara una excepci
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);        -- assigna un codi de error a la excepci
  interval_erroni EXCEPTION;                            -- declara una excepcio
  PRAGMA EXCEPTION_INIT (interval_erroni, -20002);      -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                                 -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);            -- assigna un codi de error a la excepci
  id_log NUMBER;                                        -- hi guardarem la seguent PK de la taula log
  p_valor_operacio NUMBER;                              -- hi guardarem el Calcul del valor de l'operacio
  data_fi DATE;          -- Guardara el calcul de la data final en cas que 'data_finalitzacio' sigui null
  duracio NUMBER;        -- Guardara el calcul de la durada en cas que 'durada_contracte' sigui null

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT modifica_contracte;
    
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_jugador IS NOT NULL         AND 
       p_equip_comprador IS NOT NULL AND
       p_data_signatura IS NOT NULL  THEN
  
       -- Decideix que fer segons si 'data_finalitzacio' i/o durada_contracte son null
       IF p_data_finalitzacio IS NULL AND
          p_durada_contracte IS NULL  THEN
             -- si els dos parametres son nuls es deixen tots dos nuls 
             -- (seria com un contracte fixa, de duraci indefinida)
             data_fi := NULL;
             duracio := NULL;
       ELSIF p_data_finalitzacio IS NULL THEN
          -- calcula la data de finalitzacio del contracte
          data_fi := p_data_signatura + NUMTOYMINTERVAL(p_durada_contracte,'year');
          duracio := p_durada_contracte;
       ELSIF p_durada_contracte IS NULL THEN
          -- calcula la duracio del contracte en ANYS (les fraccions d'any es guarden com a decimals)
          duracio := MONTHS_BETWEEN(p_data_finalitzacio , p_data_signatura)/12;
          data_fi := p_data_finalitzacio;
       ELSIF p_data_finalitzacio IS NOT NULL AND
             p_durada_contracte IS NOT NULL  THEN
             -- calcula l'interval entre les dates d'inici i final
             duracio := ROUND(MONTHS_BETWEEN(p_data_finalitzacio , p_data_signatura)/12 , 5);
             data_fi := p_data_finalitzacio;
             -- comprova que el parametre 'durada_contracte' coincideix amb l'interval calculat
             IF duracio != p_durada_contracte THEN 
                RAISE_APPLICATION_ERROR(-20002, 'La duracio del contracte no coincideix amb l`interval de dates.'); 
             END IF;
       END IF;

       -- Calcula el valor de l'operacio.
       p_valor_operacio := p_compensacio_economica + p_sou_brut_anual * duracio;
       
       -- Guarda les dades del contracte a la taula 'contracte'.
       UPDATE contracte SET
         equip_venedor = p_equip_venedor
       , data_finalitzacio = data_fi
       , durada_contracte = duracio
       , sou_brut_anual = p_sou_brut_anual
       , compensacio_economica = p_compensacio_economica
       , valor_operacio = p_valor_operacio 
       WHERE jugador = p_jugador 
         AND equip_comprador = p_equip_comprador 
         AND TRUNC(data_signatura) = TRUNC(p_data_signatura);
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parmetres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO modifica_contracte;            -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parmetre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN interval_erroni THEN       -- En cas que la duraci no coincideixi amb el interval entre dates
        ROLLBACK TO modifica_contracte;            -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20002, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO modifica_contracte;            -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO modifica_contracte;            -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
END;

-- PASSA A GUARDAR LES DADES EN LA TAULA log
  
-- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat 
id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                   -- la data i lhora en que sha cridat al procediment
  , 'modifica_contracte'                            -- el nom del procediment cridat
  ,     ', jugador = '               || p_jugador   -- la llista de parmetres dentrada (com un llarg string)
     || ', equip_comprador = '       || p_equip_comprador
     || ', equip_venedor = '         || p_equip_venedor
     || ', sou_brut_anual = '        || p_sou_brut_anual
     || ', compensacio_economica = ' || p_compensacio_economica
     || ', data_signatura = '        || p_data_signatura
     || ', data_finalitzacio = '     || p_data_finalitzacio
     || ', durada_contracte = '      || p_durada_contracte
  , 'RSP = ' || RSP);                                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
END modifica_contracte;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment ALTA_PARTIT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet donar dalta un partit en la B.D.  
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom             tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * competicio      NUMBER    IN    Enter que representa el Id de la competici. Cal que la      Un enter que coincideixi amb el 
 *                                 competici hagi estat prviament donada dalta a la BD.	     Id de la competici
 * -------------------------------------------------------------------------------------------------------------------------------
 * equip_local     NUMBER    IN    Enter que representa el Id de lequip que juga com a local.  Un enter que coincideixi amb el 
 *                                 Cal que lequip hagi estat prviament donat dalta a la BD.	 Id de lequip que juga com a local
 * -------------------------------------------------------------------------------------------------------------------------------
 * equip_visitant  VARCHAR2  IN    Enter que representa el Id de lequip que juga com a         Un enter que coincideixi amb el 
 *                                 visitant. Cal que lequip hagi estat prviament donat         Id de lequip que juga com a 
 *                                 dalta a la BD.	                                             visitant.
 * -------------------------------------------------------------------------------------------------------------------------------
 * data_hora      TIMESTAMP  IN    Especifica la data i lhora en que es jugar el partit.      NULL
 *                                 El format de la data te que ser dd-mm-aa hh:mm:ss o be     Un string de 17 carcters 
 *                                 dd/mm/aa hh:mm:ss	                                         representant la data i lhora
 * -------------------------------------------------------------------------------------------------------------------------------
 * lloc           VARCHAR2   IN    Especifica el lloc on es jugar el partit. Es sensible a     NULL
 *                                 majscules i minscules. I tamb es sensible als accents     Un string amb el lloc (max.45 
 *                                 i altres signes de puntuaci.	                             carcters)
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2    OUT     Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.  
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.  Excepci definida per el procediment, que es 
 *                                                                              llena quan un o mes parmetres dentrada (noms 
 *                                                                              els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del    Excepci definida internament i anomenada  per el 
 *                                  constraint]) violated - parent key not      procediment, que es llena quan un dels parmetres 
 *                                  found.                                      dentrada fa referncia a una clau forana amb un 
 *                                                                              valor que no hi es. En el missatge derror 
 *                                                                              sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                         Amb aquest nom dexcepci, el procediment captura 
 *                                                                              qualsevol altre excepci no especificada abans.  
 *                                                                              El codi derror i el missatge dependran de la 
 *                                                                              causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE 
PROCEDURE alta_partit
( RSP             OUT VARCHAR2                     -- retornara el codi derror si nhi ha hagut.
, competicio      IN  NUMBER
, equip_local     IN  NUMBER
, equip_visitant  IN  NUMBER
, data_hora       IN  TIMESTAMP
, lloc            IN  VARCHAR2
, punts_local     IN  NUMBER := 0
, punts_visitant  IN  NUMBER := 0 ) IS 

  parametre_nul EXCEPTION;                         -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);   -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                            -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);       -- assigna un codi de error a la excepci
  id_partit NUMBER;                                -- hi guardarem la seguent PK de la taula partit
  id_log NUMBER;                                   -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT alta_partit;
    
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF competicio IS NOT NULL      AND 
       equip_local IS NOT NULL     AND
       equip_visitant IS NOT NULL  THEN
  
       -- Consulta la sequencia que indica el seguent valor de la clau primaria.
       id_partit := partit_seq.nextval;
       
       -- Guarda les dades del nou partit a la taula 'partit'.
       INSERT INTO partit VALUES
       ( id_partit
       , competicio
       , equip_local
       , equip_visitant
       , data_hora
       , lloc
       , punts_local
       , punts_visitant );
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO alta_partit;                   -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO alta_partit;                   -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO alta_partit;                   -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
    END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'alta_partit'                              -- el nom del procediment cridat
  ,       'competicio = '     || competicio    -- la llista de parmetres dentrada (com un llarg string)
     || ', equip_local = '    || equip_local
     || ', equip_visitant = ' || equip_visitant
     || ', data_hora = '      || data_hora
     || ', lloc = '           || lloc
     || ', punts_local = '    || punts_local
     || ', punts_visitant = ' || punts_visitant
  , 'RSP = ' || RSP );                           -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END alta_partit;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment BAIXA_PARTIT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet donar de baixa un partit en la B.D.   
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom          tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_id_partit  NUMBER   IN     Enter que representa el Id del partit. Cal que del          Un enter que coincideixi amb el 
 *                              partit hagi estat prviament donat dalta a la BD.	         Id del partit
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2    OUT     Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci. 
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                  Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.    Excepci definida per el procediment, que es 
 *                                                                                llena quan un o mes parmetres dentrada (noms 
 *                                                                                els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                           Amb aquest nom dexcepci, el procediment 
 *                                                                                captura qualsevol altre excepci no especificada 
 *                                                                                abans.  El codi derror i el missatge dependran 
 *                                                                                de la causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE 
PROCEDURE baixa_partit
( RSP          OUT VARCHAR2                             -- retornara el codi derror si nhi ha hagut.
, p_id_partit  IN  NUMBER ) IS 

  parametre_nul EXCEPTION;                              -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);        -- assigna un codi de error a la excepcio
  id_log NUMBER;                                        -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT baixa_partit;
    
    -- Comprova que les claus foranes no tinguin un valor nul.
    IF p_id_partit IS NOT NULL THEN
  
       -- Esborra el partit de la taula partit.
       DELETE FROM partit WHERE id_partit = p_id_partit;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO baixa_partit;                  -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO baixa_partit;                  -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
    END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'baixa_partit'                             -- el nom del procediment cridat
  , 'id_partit = ' || p_id_partit              -- la llista de parmetres dentrada (com un llarg string)
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END baixa_partit;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment MODIFICA_PARTIT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet modificar les dades dun partit que ja existeix en la B.D.  
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom               tipus    IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_id_partit       NUMBER     IN    Enter que representa el Id del partit. Cal que del partit     Un enter que coincideixi amb 
 *                                    hagi estat prviament donat dalta a la BD.	                 el Id del partit
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_competicio      NUMBER     IN    Enter que representa el Id de la competici. Cal que la       Un enter que coincideixi amb 
 *                                    competici hagi estat prviament donada dalta a la BD.	     el Id de la competici
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_equip_local     NUMBER     IN    Enter que representa el Id de lequip que juga com a local.   Un enter que coincideixi amb 
 *                                    Cal que lequip hagi estat prviament donat dalta a la BD.	 el Id de lequip que juga com 
 *                                                                                                   a local.
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_equip_visitant  VARCHAR2   IN    Enter que representa el Id de lequip que juga com a          Un enter que coincideixi amb 
 *                                    visitant. Cal que lequip hagi estat prviament donat          el Id de lequip que juga com 
 *                                    dalta a la BD.	                                             a visitant.
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_data_hora       TIMESTAMP  IN    Especifica la data i lhora en que es jugar el partit. El    NULL
 *                                    format de la data te que ser dd-mm-aa hh:mm:ss o be         Un string de 17 carcters 
 *                                    dd/mm/aa hh:mm:ss	                                         representant la data i lhora
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_lloc            VARCHAR2   IN    Especifica el lloc on es jugar el partit. Es sensible a      NULL
 *                                    majscules i minscules. I tamb es sensible als accents      Un string amb el lloc 
 *                                    i altres signes de puntuaci.	                                 (max.45 carcters)
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2    OUT     Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.	 Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del     Excepci definida internament i anomenada per el 
 *                                  constraint]) violated - parent key not       procediment, que es llena quan un dels 
 *                                  found.	                                     parmetres dentrada fa referncia a una clau 
 *                                                                               forana amb un valor que no hi es. En el missatge 
 *                                                                               derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE 
PROCEDURE modifica_partit
( RSP               OUT VARCHAR2                           -- retornara el codi derror si nhi ha hagut.
, p_id_partit       IN  NUMBER
, p_competicio      IN  NUMBER
, p_equip_local     IN  NUMBER
, p_equip_visitant  IN  NUMBER
, p_data_hora       IN  TIMESTAMP
, p_lloc            IN  VARCHAR2
, p_punts_local     IN  NUMBER := 0
, p_punts_visitant  IN  NUMBER := 0 ) IS 

  parametre_nul EXCEPTION;                                 -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);           -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                                    -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);               -- assigna un codi de error a la excepci
  id_log NUMBER;                                           -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT modifica_partit;
    
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_id_partit IS NOT NULL       AND 
       p_competicio IS NOT NULL      AND
       p_equip_local IS NOT NULL     AND
       p_equip_visitant IS NOT NULL  THEN
  
       -- Guarda les dades del partit a la taula 'partit'.
       UPDATE partit SET
         competicio = p_competicio
       , equip_local = p_equip_local
       , equip_visitant = p_equip_visitant
       , data_hora = p_data_hora
       , lloc = p_lloc 
       , punts_local = p_punts_local
       , punts_visitant = p_punts_visitant
       WHERE id_partit = p_id_partit;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO modifica _partit;              -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO modifica _partit;              -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO modifica _partit;              -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
    END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                -- la data i lhora en que sha cridat al procediment
  , 'modifica_partit'                           -- el nom del procediment cridat
  ,       'id_partit = '      || p_id_partit    -- la llista de parmetres dentrada (com un llarg string)
     || ', competicio = '     || p_competicio
     || ', equip_local = '    || p_equip_local
     || ', equip_visitant = ' || p_equip_visitant
     || ', data_hora = '      || p_data_hora
     || ', lloc = '           || p_lloc
     || ', punts_local = '    || p_punts_local
     || ', punts_visitant = ' || p_punts_visitant
  , 'RSP = ' || RSP );                                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END modifica_partit;
/







