/**
 * Estudiant: David Carreras Gresely
 * TFC: Disseny i implementaci de la base de dades dun sistema 
 *      de gesti d'informaci de jugadors de bsquet.
 * Data: 12-06-2013
 * Document: Creacio_Proc_ABM_dades_jugador_partit.sql
 * Contingut: Aquest document cont els scripts per crear els Procediments de
 *            Alta, Baixa i Modificaci (ABM) de dades estadstiques de jugadors
 *            en cada Partit.
 */
 
/**
 * Especificacions comunes a tots els Procediemnts:
 *
 *    Tots els procediments disposen de tractament dexcepcions per gestionar els errors mes probables. 
 *    En cas dexcepci, el procediment guarda el codi i el missatge derror en el parmetre de 
 *    sortida RSP. Si no hi ha hagut cap error el contingut de RSP ser OK. En el subapartat 
 *    Codis derror que trobarem mes avall en la descripci de cada un dels procediments, hi ha la 
 *    relaci dels errors que el procediment captura, els codis i missatges derror que el procediment 
 *    retorna, i la descripci de les causes que provoquen el llanament de lexcepci.
 *    A mes a mes, cada procediment guarda en la taula log un registre de la crida feta al procediment. 
 *    Per tal dassegurar que el registre es guardar fins i tot en cas derror, he implementat tots els 
 *    procediments creant a dins del bloc principal BEGIN...END, un sub-bloc on hi poso totes les 
 *    operacions que fa el procediment, excepte guardar el registre en la taula log. A mes a mes, el 
 *    sub-bloc captura les seves prpies excepcions. Daquesta forma, encara que es llenci una excepci 
 *    des del sub-bloc, lexecuci del procediment continuar en el bloc principal, i per tan, sempre es 
 *    guardar lerror en la taula log.
 *    
 *    Dades guardades en la taula log:
 *    ================================================================================================
 *    nom	          tipus	               descripci
 *    ================================================================================================
 *    data_hora	      TIMESTAMP            Especifica la data i lhora en que sha cridat al 
 *                                         procediment. Per defecte, tots els procediments li 
 *                                         assignen la data i lhora actual del sistema (SYSTIMESTAMP) 
 *    ------------------------------------------------------------------------------------------------
 *    procediment	  VARCHAR2(45 CHAR)    Es un string que especifica el nom del procediment cridat.
 *    ------------------------------------------------------------------------------------------------
 *    parametres_in	  VARCHAR2(2000 CHAR)  Es la llista de parmetres dentrada del procediment amb 
 *                                         els seus valors, representada com un llarg string
 *    ------------------------------------------------------------------------------------------------
 *    parametres_out  VARCHAR2(2000 CHAR)  Es la llista de parmetres de sortida del procediment 
 *                                         amb els seus valors, representada com un llarg string.
 *    ================================================================================================
 */








/**
 * -------------------------------------------------------------------------------
 * -- Procediment ALTA_MODIF_ESTD_JUGADOR_PARTIT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet entrar en la B.D.  noves dades estadstiques dun jugador en un partit, i tamb permet 
 *             modificar-les en cas que ja shagin entrat prviament. 
 *             
 *             Per tal devitar la possibilitat que es generi un error al intentar modificar unes dades que 
 *             encara no shagin introdut, he optat per utilitzar la instrucci MERGE, en un nic procediment 
 *             que serveix tan per donar dalta noves dades destadstiques de joc dun jugador, com tamb per 
 *             modificar aquestes dades en cas que ja existeixin a la BD.
 *    
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom              tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_jugador        NUMBER   IN     Enter que representa el Id del jugador. Cal que el jugador        Un enter que coincideixi 
 *                                  hagi estat prviament donat dalta a la BD.	                       amb el Id del jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_partit         NUMBER   IN     Enter que representa el Id del partit. Cal que el partit hagi     Un enter que coincideixi 
 *                                  estat prviament donat dalta a la BD.	                           amb el Id del partit
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_minuts_jugats  FLOAT    IN     Especifica el nombre de minuts que el jugador ha jugat en el      Nombre amb decimals 
 *                                  partit. La part decimal, indica una fracci de minut.	           (p.ex. 12.05)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_PT             NUMBER   IN     Especifica el nombre de punts total aconseguits per el jugador.   Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_T1I            NUMBER   IN     Especifica el nombre de llanaments lliures intentats.            Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_T1E            NUMBER   IN     Especifica el nombre de llanaments lliures encistellats          Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_T2I            NUMBER   IN     Especifica el nombre de llanaments de dos punts intentats.       Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_T2E            NUMBER   IN     Especifica el nombre de llanaments de dos punts encistellats     Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_T3I            NUMBER   IN     Especifica el nombre de llanaments de tres punts intentats.      Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_T3E            NUMBER   IN     Especifica el nombre de llanaments de tres punts encistellats    Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_RD             NUMBER   IN     Especifica el nombre de rebots defensius                          Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_RO             NUMBER   IN     Especifica el nombre de rebots ofensius                           Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_ASIS           NUMBER   IN     Especifica el nombre dassistncies                               Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_TF             NUMBER   IN     Especifica el nombre de taps a favor                              Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_TC             NUMBER   IN     Especifica el nombre de taps en contra.                           Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_PR             NUMBER   IN     Especifica el nombre de pilotes recuperades                       Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_PP             NUMBER   IN     Especifica el nombre de pilotes perdudes                          Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_FC             NUMBER   IN     Especifica el nombre de faltes comeses                            Un enter (=>0)
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_FR             NUMBER   IN     Especifica el nombre de faltes rebudes                            Un enter (=>0)
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	   IN/OUT	descripci	                                                Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2    OUT     Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK si tot ha 
 *                          lutilitzar el procediment per passar al software client     anat be.
 *                          informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror i la seva 
 *                                                                                        descripci en cas duna excepci.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del     Excepci definida internament i anomenada  per 
 *                                  constraint]) violated - parent key not       el procediment, que es llena quan un dels 
 *                                  found.	                                     parmetres dentrada fa referncia a una clau 
 *                                                                               forana amb un valor que no hi es. En el missatge 
 *                                                                               derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE alta_modif_estd_jugador_partit
( RSP              OUT VARCHAR2                 -- retornar el codi derror si nhi ha hagut.
, p_jugador        IN  NUMBER
, p_partit         IN  NUMBER
, p_minuts_jugats  IN  FLOAT  := 0
, p_PT             IN  NUMBER := 0
, p_T1I            IN  NUMBER := 0
, p_T1E            IN  NUMBER := 0
, p_T2I            IN  NUMBER := 0
, p_T2E            IN  NUMBER := 0
, p_T3I            IN  NUMBER := 0
, p_T3E            IN  NUMBER := 0
, p_RD             IN  NUMBER := 0
, p_RO             IN  NUMBER := 0
, p_ASIS           IN  NUMBER := 0
, p_TF             IN  NUMBER := 0
, p_TC             IN  NUMBER := 0
, p_PR             IN  NUMBER := 0
, p_PP             IN  NUMBER := 0
, p_FC             IN  NUMBER := 0
, p_FR             IN  NUMBER := 0 ) IS 

  parametre_nul EXCEPTION;                          -- declara una excepci
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);    -- assigna un codi de error a la excepci
  fk_erronia EXCEPTION;                             -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);        -- assigna un codi de error a la excepci
  id_log NUMBER;                                    -- hi guardarem la segent PK de la taula log
  p_valoracio NUMBER;                               -- hi guardarem el Clcul de la valoracio del jugador
  p_valoracio_ponderada NUMBER;                     -- hi guardarem el Clcul de la valoracio ponderada
  factor_p NUMBER;                                  -- hi guardarem el factor de ponderaci

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauraci per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT alta_modif_estd_jugador_partit;

    -- Comprova que no hi hagi cap parametre nul, o que no en falti cap.
    IF p_jugador IS NOT NULL AND 
       p_partit IS NOT NULL  THEN

       -- Calcula la valoracio.
       p_valoracio := p_PT+p_T1E+p_T2E+p_T3E+p_RD+p_RO+p_ASIS+p_TF+p_PR+p_FR-p_T1I-p_T2I-p_T3I-p_TC-p_PP-p_FC;
  
       -- Obt el factor de ponderaci.
       SELECT factor_p 
       INTO factor_p 
       FROM competicio c, partit p 
       WHERE p.competicio = c.id_competicio 
       AND p.id_partit = p_partit;

       -- Calcula la valoraci ponderada.
       p_valoracio_ponderada := p_valoracio * factor_p;

       MERGE INTO estadistica_jugador_partit D
         USING (SELECT 1 FROM dual ) S
         ON (D.jugador = p_jugador AND D.partit = p_partit)  -- comprova si en la taula dest, ja existeix
         WHEN MATCHED THEN                                   -- una entrada per aquest jugador i partit
           UPDATE SET D.minuts_jugats = p_minuts_jugats      -- si hi es, actualitza les dades
                    , D.PT = p_PT
                    , D.T1I = p_T1I
                    , D.T1E = p_T1E
                    , D.T2I = p_T2I
                    , D.T2E = p_T2E
                    , D.T3I = p_T3I
                    , D.T3E = p_T3E
                    , D.RD = p_RD
                    , D.RO = p_RO
                    , D.ASIS = p_ASIS
                    , D.TF = p_TF
                    , D.TC = p_TC
                    , D.PR = p_PR
                    , D.PP = p_PP
                    , D.FC = p_FC
                    , D.FR = p_FR
                    , D.valoracio = p_valoracio
                    , D.valoracio_ponderada = p_valoracio_ponderada 
           WHERE D.jugador = p_jugador AND D.partit = p_partit 
         WHEN NOT MATCHED THEN                               -- si no hi es, n'entra una de nova
           INSERT (D.jugador 
                 , D.partit
                 , D.minuts_jugats
                 , D.PT
                 , D.T1I
                 , D.T1E
                 , D.T2I
                 , D.T2E
                 , D.T3I
                 , D.T3E
                 , D.RD
                 , D.RO
                 , D.ASIS
                 , D.TF
                 , D.TC
                 , D.PR
                 , D.PP
                 , D.FC
                 , D.FR
                 , D.VALORACIO
                 , D.VALORACIO_PONDERADA) 
           VALUES (p_jugador
                 , p_partit
                 , p_minuts_jugats
                 , p_PT
                 , p_T1I
                 , p_T1E
                 , p_T2I
                 , p_T2E
                 , p_T3I
                 , p_T3E
                 , p_RD
                 , p_RO
                 , p_ASIS
                 , p_TF
                 , p_TC
                 , p_PR
                 , p_PP
                 , p_FC
                 , p_FR
                 , p_VALORACIO
                 , p_VALORACIO_PONDERADA);

    ELSE
       -- Si un dels parmetres es nul, llena una excepci
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parmetres nuls.');
    END IF;

    -- Si tot ha anat be, el parametre de sortida indicar OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                        -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO alta_modif_estd_jugador_partit;  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -20000, '||SQLERRM;           -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                           -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO alta_modif_estd_jugador_partit;  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;            -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                               -- en qualsevol altre tipus derror
        ROLLBACK TO alta_modif_estd_jugador_partit;  -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;    -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);    -- i llena una excepci.
    END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la seqncia que indica el segent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'alta_modif_estd_jugador_partit'           -- el nom del procediment cridat
  ,       'jugador = '       || p_jugador      -- la llista de parmetres dentrada (com un llarg string)
     || ', partit = '        || p_partit
     || ', minuts_jugats = ' || p_minuts_jugats
     || ', PT = '            || p_PT
     || ', T1I = '           || p_T1I
     || ', T1E = '           || p_T1E
     || ', T2I = '           || p_T2I
     || ', T2E = '           || p_T2E
     || ', T3I = '           || p_T3I
     || ', T3E = '           || p_T3E
     || ', RD  = '           || p_RD
     || ', RO  = '           || p_RO
     || ', ASIS = '          || p_ASIS
     || ', TF = '            || p_TF
     || ', TC = '            || p_TC
     || ', PR = '            || p_PR
     || ', PP = '            || p_PP
     || ', FC = '            || p_FC
     || ', FR = '            || p_FR
  , 'RSP = ' || RSP );                         -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END alta_modif_estd_jugador_partit;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment BAIXA_ESTAD_JUGADOR_PARTIT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet eliminar les dades estadstiques dun jugador en un partit.  
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom        tipus   IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_jugador  NUMBER    IN    Enter que representa el Id del jugador. Cal que el            Un enter que coincideixi amb el Id 
 *                            jugador hagi estat prviament donat dalta a la BD.	         del jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_partit   NUMBER    IN    Enter que representa el Id del partit. Cal que el partit      Un enter que coincideixi amb el Id 
 *                            hagi estat prviament donat dalta a la BD.	                 del partit
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	tipus	 IN/OUT   descripci                                                      Valors possibles
 * ===============================================================================================================================
 * RSP  VARCHAR2   OUT    Es un parmetre de tipus varchar2 que per referncia             Un string amb la paraula OK si tot 
 *                        lutilitzar el procediment per passar al software client         ha anat be.
 *                        informaci sobre si hi ha hagut una excepci o no.	           Un string amb el codi derror i la 
 *                                                                                          seva descripci en cas duna excepci.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del     Excepci definida internament i anomenada  per 
 *                                  constraint]) violated - parent key not 	     el procediment, que es llena quan un dels 
 *                                  found.                                       parmetres dentrada fa referncia a una clau 
 *                                                                               forana amb un valor que no hi es. En el missatge 
 *                                                                               derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 * 	             OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */

CREATE OR REPLACE 
PROCEDURE baixa_estad_jugador_partit
( RSP       OUT VARCHAR2                                -- retornara el codi derror si nhi ha hagut.
, p_jugador IN  NUMBER
, p_partit  IN  NUMBER ) IS 

  parametre_nul EXCEPTION;                              -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);        -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                                 -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);            -- assigna un codi de error a la excepci
  id_log NUMBER;                                        -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Estableix un punt de restauracio per si hi ha algun error i cal desfer els canvis fets
    SAVEPOINT baixa_estad_jugador_partit;
    
    -- Comprova que les claus foranes no tinguin un valor nul.
    IF p_jugador IS NOT NULL AND 
       p_partit IS NOT NULL  THEN
  
       -- Esborra el contracte de la taula 'contracte'.
       DELETE FROM estadistica_jugador_partit WHERE jugador = p_jugador AND partit = p_partit;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
    
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
  
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        ROLLBACK TO baixa_estad_jugador_partit;    -- Ho desfa tot fins al punt de restauracio
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        ROLLBACK TO baixa_estad_jugador_partit;    -- Ho desf tot fins al punt de restauraci
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        ROLLBACK TO baixa_estad_jugador_partit;    -- Guarda l'error en el parametre de sortida
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
    END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                           -- la data i lhora en que sha cridat al procediment
  , 'baixa_estad_jugador_partit'           -- el nom del procediment cridat
  ,       'jugador = ' || p_jugador        -- la llista de parmetres dentrada (com un llarg string)
     || ', partit = '  || p_partit
  , 'RSP = ' || RSP);                      -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END baixa_estad_jugador_partit;
/







