/**
 * Estudiant: David Carreras Gresely
 * TFC: Disseny i implementaci de la base de dades dun sistema 
 *      de gesti d'informaci de jugadors de bsquet.
 * Data: 12-06-2013
 * Document: Creacio_Proc_de_Consulta.sql
 * Contingut: Aquest document cont els scripts per crear els Procediments que permeten fer consultes
 *            a la BD. d'acord amb els requeriments dek TFC.
 */
 
/**
 * Especificacions comunes a tots els Procediemnts:
 *
 *    Tots els procediments disposen de tractament dexcepcions per gestionar els errors mes probables. 
 *    En cas dexcepci, el procediment guarda el codi i el missatge derror en el parmetre de 
 *    sortida RSP. Si no hi ha hagut cap error el contingut de RSP ser OK. En el subapartat 
 *    Codis derror que trobarem mes avall en la descripci de cada un dels procediments, hi ha la 
 *    relaci dels errors que el procediment captura, els codis i missatges derror que el procediment 
 *    retorna, i la descripci de les causes que provoquen el llanament de lexcepci.
 *    A mes a mes, cada procediment guarda en la taula log un registre de la crida feta al procediment. 
 *    Per tal dassegurar que el registre es guardar fins i tot en cas derror, he implementat tots els 
 *    procediments creant a dins del bloc principal BEGIN...END, un sub-bloc on hi poso totes les 
 *    operacions que fa el procediment, excepte guardar el registre en la taula log. A mes a mes, el 
 *    sub-bloc captura les seves prpies excepcions. Daquesta forma, encara que es llenci una excepci 
 *    des del sub-bloc, lexecuci del procediment continuar en el bloc principal, i per tan, sempre es 
 *    guardar lerror en la taula log.
 *    
 *    Dades guardades en la taula log:
 *    ================================================================================================
 *    nom	          tipus	               descripci
 *    ================================================================================================
 *    data_hora	      TIMESTAMP            Especifica la data i lhora en que sha cridat al 
 *                                         procediment. Per defecte, tots els procediments li 
 *                                         assignen la data i lhora actual del sistema (SYSTIMESTAMP) 
 *    ------------------------------------------------------------------------------------------------
 *    procediment	  VARCHAR2(45 CHAR)    Es un string que especifica el nom del procediment cridat.
 *    ------------------------------------------------------------------------------------------------
 *    parametres_in	  VARCHAR2(2000 CHAR)  Es la llista de parmetres dentrada del procediment amb 
 *                                         els seus valors, representada com un llarg string
 *    ------------------------------------------------------------------------------------------------
 *    parametres_out  VARCHAR2(2000 CHAR)  Es la llista de parmetres de sortida del procediment 
 *                                         amb els seus valors, representada com un llarg string.
 *    ================================================================================================
 */








/**
 * -------------------------------------------------------------------------------
 * -- Procediment LLISTAT_JUGADORS_COMPETICIO 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: Permet obtenir el llistat de tots els jugadors duna competici donada amb totes les seves dades, 
 *             incloent la data de finalitzaci de contracte actual.  
 *             Per obtenir aquesta informaci, el procediment creua les dades de les taules competicio, competeix, 
 *             equip, contracte i jugador, buscant aquells jugadors que estan en actiu, i que tinguin contractes 
 *             vigents amb equips que juguen en la competici indicada. Com a resultat de la cerca, el procediment 
 *             retorna una llista amb tota la informaci referent a cada jugador provinent de la taula jugador, junt 
 *             amb la data de finalitzaci del contracte, que prov de la taula contracte. La llista est ordenada 
 *             alfabticament a partir dels cognoms dels jugadors.
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom           tipus         IN/OUT  descripci	                                               Valors possibles
 * ===============================================================================================================================
 * p_competicio  VARCHAR2      IN      Es un parmetre de tipus varchar2 que contindr el nom       Un string amb el nom de la 
 *                                     de la competici. El parmetre es sensible a majscules       competici (p.ex.: ACB) 
 *                                     i minscules. I tamb es sensible als accents i altres 
 *                                     signes de puntuaci. 	
 * -------------------------------------------------------------------------------------------------------------------------------
 * cv_results	SYS_REFCURSOR  IN OUT  Es un parmetre de tipus CURSOR. El software client el       Per defecte, inicialment el 
 *                                     declara i el passa com a parmetre dentrada al               cursor te valor NULL.
 *                                     procediment, el qual lutilitzar per passar per 
 *                                     referncia la informaci de sortida. El software client 
 *                                     recollir aquesta informaci i finalment tancar el cursor.	
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	       tipus          IN/OUT    descripci                                                  Valors possibles
 * ===============================================================================================================================
 * RSP         VARCHAR2       OUT       Es un parmetre de tipus varchar2 que per referncia         Un string amb la paraula OK 
 *                                      lutilitzar el procediment per passar al software client     si tot ha anat be.
 *                                      informaci sobre si hi ha hagut una excepci o no.	         Un string amb el codi derror 
 *                                                                                                    i la seva descripci en cas 
 *                                                                                                    duna excepci.
 * -------------------------------------------------------------------------------------------------------------------------------
 * cv_results  SYS_REFCURSOR  IN OUT    Es un parmetre de tipus CURSOR. El procediment              Les diverses files del 
 *                                      lutilitzar per passar per referncia la informaci de       resultat de la consulta a la 
 *                                      sortida al software client.	                                  B.D. feta per el procediment.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * +100          NO_DATA_FOUND      ORA-01403: no data found                     Excepci predefinida de Oracle que es llena quan 
 *                                                                               no es retorna cap resultat.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE llistat_jugadors_competicio
( RSP             OUT VARCHAR2
, cv_results   IN OUT SYS_REFCURSOR
, p_competicio IN     VARCHAR2 ) IS

  parametre_nul EXCEPTION;                            -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);      -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                               -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);          -- assigna un codi de error a la excepci
  id_log NUMBER;                                      -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_competicio IS NOT NULL THEN
       -- Obre el cursor i consulta la llista de jugadors
       OPEN cv_results FOR 
         SELECT c.data_finalitzacio, j.* 
         FROM competicio cm, competeix cx, equip e, contracte c, jugador j 
         WHERE cm.id_competicio = cx.competicio 
         AND cx.equip = e.id_equip 
         AND e.id_equip = c.equip_comprador 
         AND c.jugador = j.id_jugador 
         AND c.data_finalitzacio > SYSDATE 
         AND j.activitat = 'actiu' 
         AND cm.nom = p_competicio 
         ORDER BY j.cognoms; 
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
      
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
    
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul, 
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN NO_DATA_FOUND THEN                      -- per el cas que no retorni cap dada
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'llistat_jugadors_competicio'              -- el nom del procediment cridat
  , 'p_competicio = ' || p_competicio          -- la llista de parmetres dentrada (com un llarg string)
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END llistat_jugadors_competicio;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment LLISTA_EQUIPS_PUNTS_TOTALS 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: A partir del nom duna competici, permet obtenir el llistat de tots els equips que participen en aquesta 
 *             competici. El procediment no sols retorna totes les dades referents a cada equip trobat, sin que a mes 
 *             a mes, de cada equip nobt la suma de tots els punts a favor que ha aconseguit al llarg de la competici 
 *             en la temporada actual, i retorna la llista dequips ordenada segons el nombre de punts obtinguts de mes 
 *             a menys.
 *             Per obtenir aquesta informaci, el procediment per una banda creua les dades de les taules competicio, 
 *             competeix, i equip triant aquells equips que participen en la competici especificada i en la 
 *             temporada actual. I per altre banda, per obtenir la suma dels punts a favor de cada un dels equips trobats, 
 *             fa una subconsulta en la que creua les dades de les taules competicio, partit, estadistica_jugador_partit, 
 *             i jugadors_convocats, recopilant els punts marcats pels jugadors en cada un dels partits de la competici.
 * 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom               tipus          IN/OUT  descripci	                                              Valors possibles
 * ===============================================================================================================================
 * p_nom_competicio  VARCHAR2       IN	    Es un parmetre de tipus varchar2 que contindr el nom     Un string amb el nom de 
 *                                          de la competici. El parmetre es sensible a majscules     la competici (p.ex. ACB)
 *                                          i minscules. I tamb es sensible als accents i altres 
 *                                          signes de puntuaci. 	 
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_equips_pt  SYS_REFCURSOR  IN OUT  Es un parmetre de tipus CURSOR. El software client el     Per defecte, inicialment 
 *                                          declara i el passa com a parmetre dentrada al             el cursor te valor NULL.
 *                                          procediment, el qual lutilitzar per passar per 
 *                                          referncia la informaci de sortida. El software client 
 *                                          recollir aquesta informaci i finalment tancar el cursor.	
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	             tipus	       IN/OUT   descripci	                                Valors possibles
 * ===============================================================================================================================
 * RSP               VARCHAR2        OUT    Es un parmetre de tipus varchar2 que per    Un string amb la paraula OK si tot ha 
 *                                          referncia lutilitzar el procediment per    anat be.
 *                                          passar al software client informaci sobre   Un string amb el codi derror i la 
 *                                          si hi ha hagut una excepci o no.             seva descripci en cas duna excepci.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_equips_pt  SYS_REFCURSOR  IN OUT  Es un parmetre de tipus CURSOR. El          Les diverses files del resultat de la 
 *                                          procediment lutilitzar per passar per       consulta a la B.D. feta per el 
 *                                          referncia la informaci de sortida al        procediment.
 *                                          software client.	
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * +100	         NO_DATA_FOUND      ORA-01403: no data found                     Excepci predefinida de Oracle que es llena quan 
 *                                                                               no es retorna cap resultat.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE llista_equips_punts_totals
( RSP                  OUT VARCHAR2
, llista_equips_pt  IN OUT SYS_REFCURSOR
, p_nom_competicio  IN     VARCHAR2 ) IS 

  parametre_nul EXCEPTION;                            -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);      -- assigna un codi de error a la excepcio
  id_log NUMBER;                                      -- hi guardarem la seguent PK de la taula log
  
BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_nom_competicio IS NOT NULL  THEN

         -- Obre el cursor i consulta la llista dequips
       OPEN llista_equips_pt FOR
       SELECT su.punts, e.*
       FROM
         (SELECT SUM(es.pt) AS punts, e.id_equip AS eq
          FROM equip e,
            jugadors_convocats jc,
            estadistica_jugador_partit es,
            partit p,
            competicio cm
          WHERE e.id_equip   = jc.equip
            AND jc.jugador   = es.jugador
            AND es.partit    = p.id_partit
            AND p.competicio = cm.id_competicio
          GROUP BY e.id_equip) su,
         equip e,
         competeix cx,
         competicio cm
       WHERE su.eq(+)                    = e.id_equip
         AND e.id_equip                  = cx.equip
         AND cx.competicio               = cm.id_competicio
         AND SUBSTR(cm.temporada, -4, 4) = TO_CHAR(SYSDATE, 'YYYY')
         AND  cm.nom = p_nom_competicio 
       ORDER BY punts DESC NULLS LAST, e.nom_equip;
        
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
      
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
    
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN NO_DATA_FOUND THEN                      -- per el cas que no retorni cap dada
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log
  
  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'llista_equips_punts_totals'               -- el nom del procediment cridat
  , 'p_nom_competicio = ' || p_nom_competicio  -- la llista de parmetres dentrada (com un llarg string)
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END llista_equips_punts_totals;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment LLISTA_MILLORS_JUGADORS_POS 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: A partir del mbit, el gnere, i la modalitat duna competici, el procediment permet obtenir per a cada posici 
 *             de joc, un llistat amb el 5 millors jugadors daquella posici, dacord amb el promig de la valoraci ponderada 
 *             que hagin obtingut al llarg dels partits, i ordenats de mes a menys. El procediment no sols obt la valoraci i 
 *             el nom i cognoms de cada jugador, sin que tamb proporciona totes les dades referents al jugador que es troben 
 *             en la taula jugador.
 *             Per obtenir aquesta informaci, fa tres consultes encadenades. Es a dir, te tres SELECT un dins de laltre, cada 
 *             un dells operant amb el resultat proporcionat per laltre SELECT que te a dins. El de dins de tot obt les 
 *             valoracions ponderades de cada jugador, segons lmbit, gnere, modalitat i posici especificats, i nobt el 
 *             promig.  Al seu torn, el SELECT del mig agafa el resultat anterior i el creua amb el conjunt de totes les dades 
 *             de cada jugador, i ordena el resultat segons la valoraci de mes a menys. Finalment, el SELECT de fora de tot, 
 *             tria les 5 primeres files del resultat anterior.
 *             Ara be, tot aix es noms per una posici de joc. Per tan, el procediment realitza un bucle en que repeteix tot 
 *             aquest procs de consulta per a totes i cada una de les posicions de joc, obtenint aix, diverses llistes dels 5 
 *             millors jugadors, una per a cada posici.
 *             Tota aquesta informaci es retornada per el procediment mitjanant ls de cursors, que des del software client 
 *             hauran passat com a parmetre dentrada/sortida en la crida al procediment.  Hi ha un cursor per cada una de les 
 *             posicions de joc, i els noms dels cursors coincideixen amb els noms de cada posici de joc. Daquesta forma, el 
 *             procediment guardar per referncia cada llista en el cursor corresponent, i el software client tamb sabr a 
 *             quina posici de joc correspon cada llista.
 * 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom         tipus          IN/OUT  descripci	                                              Valors possibles
 * ===============================================================================================================================
 * p_ambit     VARCHAR2       IN      Contindr el nom del ambit de la competici. Es sensible    	nacional 
 *                                    a majscules i minscules. I tamb es sensible als accents  	continental
 *                                    i altres signes de puntuaci. 	 
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_genere    NUMBER         IN      Contindr el Id del gnere dels jugadors/-res de la  	       1=mascul
 *                                    competici.                                                  2=femen
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_modalitat NUMBER         IN      Contindr el Id de la modalitat de la competici.            1=normal
 *                                                                                                 2=cadira de rodes
 * -------------------------------------------------------------------------------------------------------------------------------
 * base        SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada / sortida.     inicialment el cursor te valor 
 *                                    El procediment lutilitzar per passar per referncia els     NULL.
 *                                    resultats en la posicio base	
 * -------------------------------------------------------------------------------------------------------------------------------
 * escorta     SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada / sortida.     inicialment el cursor te valor 
 *                                    El procediment lutilitzar per passar per referncia els     NULL.
 *                                    resultats en la posicio escorta	
 * -------------------------------------------------------------------------------------------------------------------------------
 * aler	SYS    REFCURSOR      IN OUT  Cursor que es passa com a parmetre dentrada / sortida.     inicialment el cursor te valor 
 *                                    El procediment lutilitzar per passar per referncia els     NULL.
 *                                    resultats en la posicio aler	
 * -------------------------------------------------------------------------------------------------------------------------------
 * aler_pivot  SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada / sortida.    	inicialment el cursor te valor 
 *                                    El procediment lutilitzar per passar per referncia els     NULL.
 *                                    resultats en la posicio aler-pivot	
 * -------------------------------------------------------------------------------------------------------------------------------
 * pivot       SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada / sortida.     inicialment el cursor te valor 
 *                                    El procediment lutilitzar per passar per referncia els     NULL.
 *                                    resultats en la posicio pivot	
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	       tipus	      IN/OUT  descripci	                                 Valors possibles
 * ===============================================================================================================================
 * RSP         VARCHAR2       OUT     Es un parmetre de tipus varchar2 que per       Un string amb la paraula OK si tot ha 
 *                                    referncia lutilitzar el procediment per       anat be.
 *                                    passar al software client informaci sobre      Un string amb el codi derror i la seva 
 *                                    si hi ha hagut una excepci o no.	               descripci en cas duna excepci.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * base        SYS_REFCURSOR  IN OUT  Cursor que retorna per referncia els           Les diverses files del resultat de la 
 *                                    resultats en la posicio base	               consulta a la B.D. feta per el procediment.
 * -------------------------------------------------------------------------------------------------------------------------------
 * escorta     SYS_REFCURSOR  IN OUT  Cursor que retorna per referncia els           Les diverses files del resultat de la 
 *                                    resultats en la posicio escorta	               consulta a la B.D. feta per el procediment.
 * -------------------------------------------------------------------------------------------------------------------------------
 * aler        SYS_REFCURSOR  IN OUT  Cursor que retorna per referncia els           Les diverses files del resultat de la 
 *                                    resultats en la posicio aler	               consulta a la B.D. feta per el procediment.
 * -------------------------------------------------------------------------------------------------------------------------------
 * aler_pivot  SYS_REFCURSOR  IN OUT  Cursor que retorna per referncia els           Les diverses files del resultat de la 
 *                                    resultats en la posicio aler-pivot	           consulta a la B.D. feta per el procediment.
 * -------------------------------------------------------------------------------------------------------------------------------
 * pivot       SYS_REFCURSOR  IN OUT  Cursor que retorna per referncia els           Les diverses files del resultat de la 
 *                                    resultats en la posicio pivot	               consulta a la B.D. feta per el procediment.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.	 Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * +100          NO_DATA_FOUND      ORA-01403: no data found                     Excepci predefinida de Oracle que es llena quan 
 *                                                                               no es retorna cap resultat.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del     Excepci definida internament i anomenada per el 
 *                                  constraint]) violated - parent key not       procediment, que es llena quan un dels 
 *                                  found.	                                     parmetres dentrada fa referncia a una clau 
 *                                                                               forana amb un valor que no hi es. En el missatge 
 *                                                                               derror sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS		                                                     Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE llista_millors_jugadors_pos
( RSP                  OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, base              IN OUT SYS_REFCURSOR            -- retornara els resultats en la posicio base
, escorta           IN OUT SYS_REFCURSOR            -- retornara els resultats en la posicio escorta
, aler              IN OUT SYS_REFCURSOR            -- retornara els resultats en la posicio aler
, aler_pivot        IN OUT SYS_REFCURSOR            -- retornara els resultats en la posicio aler-pivot
, pivot             IN OUT SYS_REFCURSOR            -- retornara els resultats en la posicio pivot
, p_ambit           IN     VARCHAR2 
, p_genere          IN     NUMBER 
, p_modalitat       IN     NUMBER ) IS 

  parametre_nul EXCEPTION;                          -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);    -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                             -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);        -- assigna un codi de error a la excepci
  id_log NUMBER;                                    -- hi guardarem la seguent PK de la taula log
  p_posicio NUMBER;                                 -- guarda temporalment el Id de la posicio de joc
  p_nom_pos VARCHAR2(20);                           -- guarda temporalment el nom de la posicio de joc
  llista_jugadors_p SYS_REFCURSOR;                  -- guarda temporalment els resultats obtinguts en cada posicio de joc
  
  CURSOR c1 is                                      -- declara un cursor que recull la llista de 
    SELECT id_posicio_joc, descripcio               -- posicions de joc.
    FROM posicio_joc 
    ORDER BY id_posicio_joc DESC;                     
  
BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_ambit IS NOT NULL      AND 
       p_genere IS NOT NULL     AND
       p_modalitat IS NOT NULL  THEN

       OPEN c1;                                     -- obre el cursor de les posicions de joc.
       LOOP                                         -- inicia un bucle que va recollint el Id i el nom
         FETCH c1 INTO p_posicio, p_nom_pos;        -- de cada posicio de joc.
         EXIT WHEN c1%NOTFOUND;  
        
         -- segons quina sigui la posicio de joc, redirigeix el cursor temporal cap a la direccio de
         -- memoria del cursor del parametre de sortida corresponent.
         CASE
           WHEN p_posicio = 1 THEN llista_jugadors_p := base;
           WHEN p_posicio = 2 THEN llista_jugadors_p := escorta;
           WHEN p_posicio = 3 THEN llista_jugadors_p := aler;
           WHEN p_posicio = 4 THEN llista_jugadors_p := aler_pivot;
           WHEN p_posicio = 5 THEN llista_jugadors_p := pivot;
           ELSE llista_jugadors_p := llista_jugadors_p;
         END CASE;
         
         -- Obre el cursor temporal i consulta la llista de jugadors. El resultat es guardar en la zona
         -- de memria del cursor del parmetre de sortida cap a on sha redirigit abans el cursor temporal
         -- Hi ha 3 selects un dins de laltre. El de dins de tot obte la valoracio de cada jugador.
         -- El select del mig creua el resultat anterior amb el conjunt de totes les dades de cada 
         -- jugador, i ordena el resultat segons la valoracio de mes a menys. 
         -- I el select de fora de tot tria les 5 primeres files del resultat anterior.
         OPEN llista_jugadors_p FOR
         SELECT * FROM 
          (SELECT valoracio, j.* FROM 
           (SELECT AVG(es.valoracio_ponderada) AS valoracio, j.id_jugador AS jg 
            FROM jugador j,
              estadistica_jugador_partit es,
              partit p,
              competicio cm
            WHERE j.id_jugador  = es.jugador 
              AND es.partit     = p.id_partit 
              AND p.competicio  = cm.id_competicio 
              AND cm.ambit      = p_ambit
              AND cm.GENERE     = p_genere
              AND cm.MODALITAT  = p_modalitat 
              AND j.posicio_joc = p_posicio 
            GROUP BY j.id_jugador) av, 
            jugador j
          WHERE av.jg = j.id_jugador 
          ORDER BY valoracio DESC) 
         WHERE ROWNUM <= 5;
        
        END LOOP;                        -- acaba el bucle
       CLOSE c1;                         -- i tanca el cursor de les posicions de joc
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
      
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
    
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN NO_DATA_FOUND THEN                      -- per el cas que no retorni cap dada
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log
  
  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'llista_millors_jugadors_pos'              -- el nom del procediment cridat
  ,       'p_ambit = '     || p_ambit          -- la llista de parmetres dentrada (com un llarg string)
     || ', p_genere = '    || p_genere
     || ', p_modalitat = ' || p_modalitat
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END llista_millors_jugadors_pos;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment LLISTAT_CONTRACTES_REP 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: A partir del nom i cognoms dun representant de jugadors i dun any concret, el procediment llistat_contractes_rep 
 *             permet obtenir el nombre i el llistat dels contractes signats per els jugadors que ell representa durant lany 
 *             indicat. En el llistat retornat, de cada contracte sindiquen el nom i cognoms del jugador, el nom del equip 
 *             comprador, la data de signatura del contracte, i el valor total de loperaci.
 *             Per aconseguir aquesta informaci, el procediment realitza dos consultes independents entre si. Per una banda, per 
 *             trobar el nombre de contractes signats, creua les taules contracte, jugador i representant, cercant tots el 
 *             contractes signats del representant i any especificats, i contant el nombre de files retornades.
 *             Per altre banda, per obtenir la llista del contractes signats, el procediment creua les taules contracte,  
 *             jugador, representant i equip, triant noms les dades necessries dels contractes signats del representant i 
 *             any especificats. 
 *             Tota aquesta informaci es retornada per el procediment en dos parts. Per un costat, retorna el nombre de contractes 
 *             signats mitjanant el parmetre de sortida nombre_contractes. I per altre costat, retorna la llista de contractes 
 *             mitjanant el parmetre dentrada / sortida llista_contractes, que es de tipus cursor. 
 * 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom                tipus          IN/OUT  descripci	                                              Valors possibles
 * ===============================================================================================================================
 * p_nom_repres       VARCHAR2       IN      Especifica el nom del representant. Es sensible a          Un String
 *                                           majscules i minscules. I tamb es sensible als accents 
 *                                           i altres signes de puntuaci. 	 
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_cognom_repres    VARCHAR2       IN      Especifica els cognoms del representant. Es sensible a     Un String
 *                                           majscules i minscules. I tamb es sensible als accents 
 *                                           i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_any              NUMBER         IN	     Especifica lany en que shan signat els contractes. Es    Un nombre enter que 
 *                                           sensible a majscules i minscules. I tamb es sensible     representa lany amb 4 
 *                                           als accents i altres signes de puntuaci.	                 dgits  ( YYYY )
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_contractes  SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada/sortida.     inicialment el cursor te 
 *                                           El procediment lutilitzar per passar per referncia       valor NULL.
 *                                           la llista de contractes signats	
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	              tipus	         IN/OUT   descripci	                               Valors possibles
 * ===============================================================================================================================
 * RSP                VARCHAR2       OUT      Es un parmetre de tipus varchar2 que per     Un string amb la paraula OK si tot 
 *                                            referncia lutilitzar el procediment per     ha anat be.
 *                                            passar al software client informaci sobre    Un string amb el codi derror i la  
 *                                            si hi ha hagut una excepci o no.	             seva descripci en cas duna excepci. 
 * 
 * -------------------------------------------------------------------------------------------------------------------------------
 * nombre_contractes  NUMBER         OUT	 retornara el nombre de contractes signats      Un enter igual o mes gran que zero.
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_contractes  SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre            Les diverses files del resultat de 
 *                                           dentrada / sortida. El procediment             la consulta a la B.D. feta per el 
 *                                           lutilitzar per passar per referncia          procediment.
 *                                           la llista de contractes signats	
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * +100          NO_DATA_FOUND      ORA-01403: no data found                     Excepci predefinida de Oracle que es llena quan 
 *                                                                               no es retorna cap resultat.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS		                                                     Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE llistat_contractes_rep
( RSP                  OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, nombre_contractes    OUT NUMBER                   -- retornara el n de contractes
, llista_contractes IN OUT SYS_REFCURSOR            -- retornara la llista de contractes
, p_nom_repres      IN     VARCHAR2                 -- especifica el nom del representant
, p_cognom_repres   IN     VARCHAR2                 -- especifica els cognoms del representant
, p_any             IN     NUMBER ) IS              -- especifica lany en que shan signat els contractes
	
  parametre_nul EXCEPTION;                          -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);    -- assigna un codi de error a la excepcio
  id_log NUMBER;                                    -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_nom_repres    IS NOT NULL AND 
       p_cognom_repres IS NOT NULL AND 
       p_any IS NOT NULL           THEN

       -- Consulta el nombre de contractes
       SELECT COUNT(c.data_signatura) AS num_contr 
       INTO nombre_contractes 
       FROM contracte c, jugador j, representant r 
       WHERE c.jugador = j.id_jugador 
         AND j.representant = r.id_representant
         AND r.nom = p_nom_repres 
         AND r.cognoms = p_cognom_repres 
         AND TO_CHAR(c.data_signatura,'YYYY') = p_any;

       -- consulta la llista de contractes i els seus valors
       OPEN llista_contractes FOR
         SELECT j.nom||' '||j.cognoms AS jugador, 
                e.nom_equip AS equip_comprador, 
                c.data_signatura, 
                c.valor_operacio 
         FROM contracte c, jugador j, representant r, equip e  
         WHERE c.jugador = j.id_jugador 
           AND j.representant = r.id_representant 
           AND c.equip_comprador = e.id_equip 
           AND r.nom = p_nom_repres 
           AND r.cognoms = p_cognom_repres 
           AND TO_CHAR(c.data_signatura,'YYYY') = p_any;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
      
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';

    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN NO_DATA_FOUND THEN                      -- per el cas que no retorni cap dada
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log
  
  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                                 -- la data i lhora en que sha cridat al procediment
  , 'llistat_contractes_rep'                     -- el nom del procediment cridat
  ,       'p_nom_repres = '    || p_nom_repres   -- la llista de parmetres dentrada (com un llarg string)
     || ', p_cognom_repres = ' || p_cognom_repres
     || ', p_any = '           || p_any
  ,       'RSP = '               || RSP          -- la llista de parmetres de sortida
     || ', nombre_contractes = ' || nombre_contractes);
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END llistat_contractes_rep; 
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment LLISTA_EQUIPS_MES_GASTADORS 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: A partir dun any concret, el procediment permet obtenir el llistat dels 10 equips que mes diners han gastat en 
 *             adquisici de jugadors durant lany indicat. En el llistat retornat, de cada equip sindiquen el identificador 
 *             de lequip, el nom de lequip, i la suma dels valors totals de les operacions de compra realitzades. La llista 
 *             est ordenada segons el import total gastat, de mes a menys.
 *             Per aconseguir aquesta informaci, el procediment realitza dos consultes encadenades. Es a dir, te dos SELECT un 
 *             dins de laltre, el de fora operant amb el resultat proporcionat per laltre SELECT que te a dins.
 *             El de dins busca tots els contractes realitzats durant lany indicat, calcula la suma de les operacions de compra 
 *             de cada equip, junt amb el nom del equip, i nordena el resultat. Per la seva banda, el SELECT de fora de tot, 
 *             tria les 10 primeres files del resultat anterior.
 *             Tota aquesta informaci es retornada per el procediment mitjanant el parmetre dentrada / sortida llista_equips, 
 *             que es de tipus cursor.
 * 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom            tipus          IN/OUT  descripci	                                              Valors possibles
 * ===============================================================================================================================
 * p_any          NUMBER         IN      Especifica lany en que shan signat els contractes.      Un nombre enter que representa 
 *                                                                                                  lany amb 4 dgits  ( YYYY )
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_equips  SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada/sortida.    inicialment el cursor te valor 
 *                                       El procediment lutilitzar per passar per referncia      NULL.
 *                                       la llista dequips	
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	          tipus	         IN/OUT   descripci	                                Valors possibles
 * ===============================================================================================================================
 * RSP            VARCHAR2       OUT      Es un parmetre de tipus varchar2 que per      Un string amb la paraula OK si tot ha 
 *                                        referncia lutilitzar el procediment per      anat be.
 *                                        passar al software client informaci sobre     Un string amb el codi derror i la seva 
 *                                        si hi ha hagut una excepci o no.	              descripci en cas duna excepci. 
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_equips  SYS_REFCURSOR  IN OUT   Cursor que es passa com a parmetre dentrada  Les diverses files del resultat de la 
 *                                        / sortida. El procediment lutilitzar per      consulta a la B.D. feta per el 
 *                                        passar per referncia la llista dequips	      procediment.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * +100          NO_DATA_FOUND      ORA-01403: no data found                     Excepci predefinida de Oracle que es llena quan 
 *                                                                               no es retorna cap resultat.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE llista_equips_mes_gastadors
( RSP              OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, llista_equips IN OUT SYS_REFCURSOR            -- retornara la llista dequips
, p_any         IN     NUMBER ) IS              -- especifica lany

  parametre_nul EXCEPTION;                           -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);     -- assigna un codi de error a la excepcio
  id_log NUMBER;                                     -- hi guardarem la seguent PK de la taula log

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_any IS NOT NULL THEN

       -- consulta la llista d'equips, els ordena i els tria
       OPEN llista_equips FOR
       SELECT * FROM 
        (SELECT e.id_equip, e.nom_equip, SUM(c.valor_operacio) AS despesa 
         FROM equip e, contracte c 
         WHERE c.equip_comprador = e.id_equip 
         AND TO_CHAR(c.data_signatura,'YYYY') = p_any 
         GROUP BY e.id_equip, e.nom_equip 
         ORDER BY despesa DESC) 
       WHERE ROWNUM <= 10;
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
      
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
    
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN NO_DATA_FOUND THEN                      -- per el cas que no retorni cap dada
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
  
  -- PASSA A GUARDAR LES DADES EN LA TAULA log
  
  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'llista_equips_mes_gastadors'              -- el nom del procediment cridat
  , 'p_any = ' || p_any                        -- la llista de parmetres dentrada (com un llarg string)
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END llista_equips_mes_gastadors;
/








/**
 * -------------------------------------------------------------------------------
 * -- Procediment LLISTA_JUGADORS_FI_CONTRACTE 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: A partir dun pais concret, un gnere i una modalitat de competici, el procediment permet obtenir el llistat dels 
 *             jugadors que acaben el contracte durant la temporada actual, junt tamb amb el jugadors que tot i estar en actiu, 
 *             temporalment no estan contractats per cap equip.
 *             Per obtenir aquesta informaci, el procediment uneix els resultats de dos SELECT diferents. El primer creua les 
 *             taules jugador, contracte, equip, competeix i competici, buscant els jugadors que acaben el contracte al 
 *             final de la temporada actual i que juguen en les competicions del pais, gnere i modalitat especificades. Al seu 
 *             torn, laltra SELECT busca els jugadors del gnere i modalitat de joc especificats, que no tenen cap contracte que 
 *             per les dates, estigui vigent.
 *             Tota aquesta informaci es retornada per el procediment mitjanant el parmetre dentrada/sortida llista_jugadors, 
 *             que es de tipus cursor. 
 * 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * Parmetres dentrada:
 * ===============================================================================================================================
 * nom               tipus          IN/OUT  descripci	                                              Valors possibles
 * ===============================================================================================================================
 * p_pais            VARCHAR2       IN      Especifica el pais de la competici. Es sensible a         Un string amb el nom del 
 *                                          majscules i minscules. I tamb es sensible als            pais
 *                                          accents i altres signes de puntuaci.	
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_genere          NUMBER         IN      Especifica el Id del gnere de la competici.              1=mascul
 *                                                                                                     2=femen
 * -------------------------------------------------------------------------------------------------------------------------------
 * p_modalitat       NUMBER         IN      Especifica el Id de la modalitat de la competici.         1=normal
 *                                                                                                     2=cadira de rodes
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_ jugadors  SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada/sortida.     inicialment el cursor te 
 *                                          El procediment lutilitzar per passar per referncia       valor NULL.
 *                                          la llista de jugadors	
 * ===============================================================================================================================
 * 
 * 
 * 
 * Parmetres de sortida:
 * ===============================================================================================================================
 * nom	            tipus	      IN/OUT   descripci	                                              Valors possibles
 * ===============================================================================================================================
 * RSP              VARCHAR2       OUT     Es un parmetre de tipus varchar2 que per referncia    Un string amb la paraula OK 
 *                                         lutilitzar el procediment per passar al software       si tot ha anat be.
 *                                         client informaci sobre si hi ha hagut una excepci 	   Un string amb el codi derror 
 *                                         o no.                                                    i la seva descripci en cas 
 *                                                                                                  duna excepci.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * llista_jugadors  SYS_REFCURSOR  IN OUT  Cursor que es passa com a parmetre dentrada/sortida. 	Les diverses files del resultat 
 *                                         El procediment lutilitzar per passar per referncia    de la consulta a la B.D. feta 
 *                                         la llista de jugador	                                    per el procediment.
 * ===============================================================================================================================
 * 
 * 
 * 
 * El procediment pot llenar i capturar les excepcions de la segent taula. En tots el casos, el codi derror i el missatge es 
 * guardaran en el parmetre de sortida RSP, el contingut del qual al seu torn, tamb ser guardat en el registre de la taula log. 
 * 
 * Codis derror:
 * ===============================================================================================================================
 * codi derror	 Nom de lexcepci	Missatge derror retornat	                 Quan es llena
 * ===============================================================================================================================
 * -20000        PARAMETRE_NUL      ORA-20000: Hi ha un o mes parametres nuls.   Excepci definida per el procediment, que es 
 *                                                                               llena quan un o mes parmetres dentrada (noms 
 *                                                                               els IN) es NULL, o un string buit, o no hi es.  
 * -------------------------------------------------------------------------------------------------------------------------------
 * +100          NO_DATA_FOUND      ORA-01403: no data found	                 Excepci predefinida de Oracle que es llena quan 
 *                                                                               no es retorna cap resultat.
 * -------------------------------------------------------------------------------------------------------------------------------
 * -2291         FK_ERRONIA         ORA-2291: integrity constraint ([nom del     Excepci definida internament i anomenada  per el 
 *                                  constraint]) violated - parent key not       procediment, que es llena quan un dels parmetres 
 *                                  found.	                                     dentrada fa referncia a una clau forana amb un 
 *                                                                               valor que no hi es. En el missatge derror 
 *                                                                               sindica el nom de la restricci.
 * -------------------------------------------------------------------------------------------------------------------------------
 *               OTHERS                                                          Amb aquest nom dexcepci, el procediment captura 
 *                                                                               qualsevol altre excepci no especificada abans.  
 *                                                                               El codi derror i el missatge dependran de la 
 *                                                                               causa del error.
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
PROCEDURE llista_jugadors_fi_contracte
( RSP             OUT VARCHAR2                 -- retornara el codi derror si nhi ha hagut.
, llista_jugadors OUT SYS_REFCURSOR            -- retornara la llista de jugadors
, p_pais          IN  VARCHAR2                 -- especifica el pais
, p_genere        IN  NUMBER                   -- especifica el Id del genere 
, p_modalitat     IN  NUMBER ) IS              -- especifica el Id de la modalitat de joc

  parametre_nul EXCEPTION;                              -- declara una excepcio
  PRAGMA EXCEPTION_INIT (parametre_nul, -20000);        -- assigna un codi de error a la excepcio
  fk_erronia EXCEPTION;                                 -- declara una excepci
  PRAGMA EXCEPTION_INIT (fk_erronia, -2291);            -- assigna un codi de error a la excepci
  id_log NUMBER;                                        -- hi guardarem la seguent PK de la taula log
  p_activitat VARCHAR2(20) := 'actiu';                  -- especifica que el jugador estigui en actiu

BEGIN
  -- A continuaci comeno un sub-bloc on hi poso totes les operacions que fa el procediment, excepte 
  -- guardar el registre en la taula log. A mes a mes, el sub-bloc captura les seves prpies excepcions.
  -- Daquesta forma, encara que es llenci una excepci, sempre es guardar lerror en la taula log.
  BEGIN
    -- Comprova que no hi hagi cap parametre null, o que no en falti cap.
    IF p_pais IS NOT NULL       AND 
       p_genere IS NOT NULL     AND
       p_modalitat IS NOT NULL  THEN

       -- consulta la llista de jugadors
       OPEN llista_jugadors FOR
       SELECT j.* 
       FROM jugador j,
         contracte c,
         equip e,
         competeix cx,
         competicio cm
       WHERE j.id_jugador                         = c.jugador
         AND c.equip_comprador                    = e.id_equip
         AND e.id_equip                           = cx.equip
         AND cx.competicio                        = cm.id_competicio
         AND cm.pais                              = p_pais 
         AND cm.genere                            = p_genere 
         AND cm.modalitat                         = p_modalitat 
         AND SUBSTR(cm.temporada, -4, 4)          = TO_CHAR(SysDate, 'YYYY')
         AND TO_CHAR(c.data_finalitzacio, 'YYYY') = SUBSTR(cm.temporada, -4, 4) 
       UNION ALL
       SELECT j.* 
       FROM jugador j 
       WHERE j.activitat = p_activitat 
         AND j.genere    = p_genere 
         AND j.modalitat = p_modalitat 
         AND j.id_jugador NOT IN 
             (SELECT c.jugador FROM contracte c WHERE c.data_finalitzacio > SysDate);
    ELSE
       -- Si un dels parametres es null, llena una excepcio
       RAISE_APPLICATION_ERROR(-20000, 'Hi ha un o mes parametres nuls.');
    END IF;
      
    -- Si tot ha anat be, el parametre de sortida indicara OK.	
    RSP := 'OK';
    
    EXCEPTION                                    
      WHEN parametre_nul THEN                      -- En cas que un dels parmetres sigui nul,
        RSP := 'ERROR: -20000, '||SQLERRM;         -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN NO_DATA_FOUND THEN                      -- per el cas que no retorni cap dada
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN fk_erronia THEN                         -- En cas que un dels parmetres sigui una FK errnia,
        RSP := 'ERROR: -2291, '||SQLERRM;          -- Indica l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
      WHEN others THEN                             -- en qualsevol altre tipus derror
        RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;  -- Guarda l'error en el parametre de sortida
        DBMS_OUTPUT.PUT_LINE (RSP);
        RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);  -- i llena una excepci.
  END;
    
  -- PASSA A GUARDAR LES DADES EN LA TAULA log

  -- Consulta la sequencia que indica el seguent valor de la PK de la taula log i assigna el valor trobat
  id_log := log_seq.nextval;
  
  -- Guarda les dades del procediment a la taula 'log'.
  INSERT INTO log VALUES
  ( id_log
  , SYSTIMESTAMP                               -- la data i lhora en que sha cridat al procediment
  , 'llista_jugadors_fi_contracte'             -- el nom del procediment cridat
  ,       'p_pais = '      || p_pais           -- la llista de parmetres dentrada (com un llarg string)
     || ', p_genere = '    || p_genere
     || ', p_modalitat = ' || p_modalitat
  , 'RSP = ' || RSP);                          -- la llista de parmetres de sortida
  
  EXCEPTION
    WHEN others THEN                       -- captura lexcepci en cas derror en laccs a la taula log
    RSP := 'ERROR: '||SQLCODE||', '||SQLERRM;
    DBMS_OUTPUT.PUT_LINE (RSP);
    RAISE_APPLICATION_ERROR(SQLCODE,SQLERRM);
  
END llista_jugadors_fi_contracte;



