/**
 * Estudiant: David Carreras Gresely
 * TFC: Disseny i implementaci de la base de dades dun sistema 
 *      de gesti d'informaci de jugadors de bsquet.
 * Data: 12-06-2013
 * Document: Creacio_Taules_BD.sql
 * Contingut: Aquest document cont els scripts per crear les taules de la BD.
 *            Lordre en que es mostren coincideix amb lordre en que shan de 
 *            crear les taules per tal de no provocar cap conflicte amb les 
 *            claus foranes.
 *            En les taules en que la clau primria (PK) es una seqncia 
 *            numrica prpia, junt amb lscript per crear la taula shi 
 *            acompanya lscript per crear la seqncia que en el SGBD de 
 *            Oracle, permet saber quin ser el valor que haur de tenir 
 *            la PK en el prxim INSERT.
 */


-------------------------------------------------------------------------------
-- Taula REPRESENTANT
-------------------------------------------------------------------------------

CREATE TABLE representant 
( id_representant NUMBER NOT NULL
, nom VARCHAR2(20)
, cognoms VARCHAR2(45)
, dni VARCHAR2(9)
, CONSTRAINT pk_representant PRIMARY KEY(id_representant));

CREATE SEQUENCE representant_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula POSICI_JOC
-------------------------------------------------------------------------------

CREATE TABLE posicio_joc 
( id_posicio_joc NUMBER NOT NULL
, descripcio VARCHAR2(20)
, CONSTRAINT pk_posicio_joc PRIMARY KEY(id_posicio_joc));

CREATE SEQUENCE posicio_joc_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula GNERE
-------------------------------------------------------------------------------

CREATE TABLE genere
( id_genere NUMBER NOT NULL 
, nom       VARCHAR2(45 BYTE)
, CONSTRAINT pk_genere PRIMARY KEY (id_genere));

CREATE SEQUENCE genere_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula MODALITAT
-------------------------------------------------------------------------------

CREATE TABLE modalitat
( id_modalitat NUMBER NOT NULL
, nom          VARCHAR2(45 BYTE)
, CONSTRAINT pk_modalitat PRIMARY KEY (id_modalitat));

CREATE SEQUENCE modalitat_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula JUGADOR
-------------------------------------------------------------------------------

CREATE TABLE jugador 
( id_jugador NUMBER NOT NULL
, representant NUMBER NOT NULL
, posicio_joc NUMBER NOT NULL
, genere NUMBER NOT NULL
, modalitat NUMBER NOT NULL
, nacionalitat VARCHAR2(20 CHAR)
, federacio VARCHAR2(20 CHAR)
, num_federat VARCHAR2(20 CHAR)
, nom VARCHAR2(20 CHAR)
, cognoms VARCHAR2(45 CHAR)
, data_naixement DATE
, altura FLOAT
, pes FLOAT
, url_web HttpUriType
, url_video HttpUriType
, activitat VARCHAR2(7 CHAR)
, estat VARCHAR2(5 CHAR)
, sub_estat VARCHAR2(26 CHAR)
, diagnostic_actual VARCHAR2(2000 CHAR)
, data_disponibilitat DATE
, CONSTRAINT pk_jugador PRIMARY KEY (id_jugador)
, CONSTRAINT fk_jugador_1 FOREIGN KEY (representant)
  REFERENCES representant (id_representant)
, CONSTRAINT fk_jugador_2 FOREIGN KEY (posicio_joc)
  REFERENCES posicio_joc (id_posicio_joc)
, CONSTRAINT fk_jugador_3 FOREIGN KEY (genere)
  REFERENCES genere (id_genere)
, CONSTRAINT fk_jugador_4 FOREIGN KEY (modalitat)
  REFERENCES modalitat (id_modalitat)
, CONSTRAINT jug_activitat CHECK (activitat IN ('actiu', 'retirat'))
, CONSTRAINT jug_estat CHECK (estat IN ('alta', 'baixa'))
, CONSTRAINT jug_sub_estat CHECK (sub_estat IN ('baixa medica', 'baixa per motius personals')));

CREATE SEQUENCE jugador_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula EQUIP:
-------------------------------------------------------------------------------

CREATE TABLE equip 
( id_equip NUMBER NOT NULL
, nom_equip VARCHAR2(255)
, nom_representant VARCHAR2(255)
, municipi VARCHAR2(255)
, pais VARCHAR2(45)
, adreca VARCHAR2(255)
, telefon NUMBER
, url_web HttpUriType
, tipus_entitat VARCHAR2(45)
, num_socis NUMBER
, CONSTRAINT pk_equip PRIMARY KEY (id_equip));

CREATE SEQUENCE equip_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula CONTRACTE:
-------------------------------------------------------------------------------

CREATE TABLE contracte 
( jugador NUMBER NOT NULL
, equip_comprador NUMBER NOT NULL
, equip_venedor NUMBER
, data_signatura DATE NOT NULL
, data_finalitzacio DATE
, durada_contracte NUMBER(38,5)
, sou_brut_anual NUMBER(38,3)
, compensacio_economica NUMBER(38,3)
, valor_operacio NUMBER(38,3)
, CONSTRAINT pk_contracte PRIMARY KEY (jugador,equip_comprador,data_signatura)
, CONSTRAINT fk_contracte_1 FOREIGN KEY (jugador)
  REFERENCES jugador (id_jugador)
, CONSTRAINT fk_contracte_2 FOREIGN KEY (equip_comprador)
  REFERENCES equip (id_equip)
, CONSTRAINT fk_contracte_3 FOREIGN KEY (equip_venedor)
  REFERENCES equip (id_equip));



-------------------------------------------------------------------------------
-- Taula CATEGORIA:
-------------------------------------------------------------------------------

CREATE TABLE categoria 
( id_categoria NUMBER NOT NULL
, nom VARCHAR2(20)
, CONSTRAINT pk_categoria PRIMARY KEY(id_categoria));

CREATE SEQUENCE categoria_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula COMPETICI:
-------------------------------------------------------------------------------

CREATE TABLE competicio 
( id_competicio NUMBER NOT NULL
, categoria NUMBER NOT NULL
, nom VARCHAR2(20)
, ambit VARCHAR2(20)
, genere NUMBER NOT NULL
, modalitat NUMBER NOT NULL
, pais VARCHAR2(20)
, nombre_arbitres NUMBER(1)
, temporada VARCHAR2(20)
, factor_p FLOAT
, CONSTRAINT pk_competicio PRIMARY KEY (id_competicio)
, CONSTRAINT fk_competicio_1 FOREIGN KEY (categoria)
  REFERENCES categoria (id_categoria)
, CONSTRAINT fk_competicio_2 FOREIGN KEY (genere)
  REFERENCES genere (id_genere)
, CONSTRAINT fk_competicio_3 FOREIGN KEY (modalitat)
  REFERENCES modalitat (id_modalitat));

CREATE SEQUENCE competicio_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula COMPETEIX:
-------------------------------------------------------------------------------

CREATE TABLE competeix 
( competicio NUMBER NOT NULL
, equip NUMBER NOT NULL
, CONSTRAINT pk_competeix PRIMARY KEY (competicio,equip)
, CONSTRAINT fk_competeix_1 FOREIGN KEY (competicio)
  REFERENCES competicio (id_competicio)
, CONSTRAINT fk_competeix_2 FOREIGN KEY (equip)
  REFERENCES equip (id_equip));



-------------------------------------------------------------------------------
-- Taula PARTIT:
-------------------------------------------------------------------------------

CREATE TABLE partit 
( id_partit NUMBER NOT NULL
, competicio NUMBER NOT NULL
, equip_local NUMBER NOT NULL
, equip_visitant NUMBER NOT NULL
, data_hora TIMESTAMP
, lloc VARCHAR2(45 CHAR)
, CONSTRAINT pk_partit PRIMARY KEY (id_partit)
, CONSTRAINT fk_partit_1 FOREIGN KEY (competicio)
  REFERENCES competicio (id_competicio)
, CONSTRAINT fk_partit_2 FOREIGN KEY (equip_local)
  REFERENCES equip (id_equip)
, CONSTRAINT fk_partit_3 FOREIGN KEY (equip_visitant)
  REFERENCES equip (id_equip));

CREATE SEQUENCE partit_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula ESTADSTICA_JUGADOR_PARTIT:
-------------------------------------------------------------------------------

CREATE TABLE estadistica_jugador_partit 
( jugador NUMBER NOT NULL
, partit NUMBER NOT NULL
, minuts_jugats FLOAT
, pt NUMBER
, t1i NUMBER
, t1e NUMBER
, t2i NUMBER
, t2e NUMBER
, t3i NUMBER
, t3e NUMBER
, rd NUMBER
, ro NUMBER
, asis NUMBER
, tf NUMBER
, tc NUMBER
, pr NUMBER
, pp NUMBER
, fc NUMBER
, fr NUMBER
, valoracio NUMBER
, valoracio_ponderada NUMBER
, CONSTRAINT pk_est_jp PRIMARY KEY (jugador,partit)
, CONSTRAINT fk_eest_jp_1 FOREIGN KEY (jugador)
  REFERENCES jugador (id_jugador)
, CONSTRAINT fk_est_jp_2 FOREIGN KEY (partit)
REFERENCES partit (id_partit));



-------------------------------------------------------------------------------
-- Taula JUGADORS_CONVOCATS:
-------------------------------------------------------------------------------

CREATE TABLE jugadors_convocats 
( partit NUMBER NOT NULL
, equip NUMBER NOT NULL
, jugador NUMBER NOT NULL
, CONSTRAINT pk_jugadors_convocats PRIMARY KEY (partit,equip,jugador)
, CONSTRAINT fk_jugadors_convocats_1 FOREIGN KEY (partit)
  REFERENCES partit (id_partit)
, CONSTRAINT fk_jugadors_convocats_2 FOREIGN KEY (equip)
  REFERENCES equip (id_equip)
, CONSTRAINT fk_jugadors_convocats_3 FOREIGN KEY (jugador)
  REFERENCES jugador (id_jugador));



-------------------------------------------------------------------------------
-- Taula RBITRE:
-------------------------------------------------------------------------------

CREATE TABLE arbitre 
( id_arbitre NUMBER NOT NULL
, categoria NUMBER NOT NULL
, nom VARCHAR2(20)
, cognoms VARCHAR2(45)
, num_colegiat VARCHAR2(20)
, CONSTRAINT pk_arbitre PRIMARY KEY (id_arbitre)
, CONSTRAINT fk_arbitre_1 FOREIGN KEY (categoria)
  REFERENCES categoria (id_categoria));

CREATE SEQUENCE arbitre_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula TIPUS_RBITRE:
-------------------------------------------------------------------------------

CREATE TABLE tipus_arbitre 
( id_tipus_arb NUMBER NOT NULL
, nom VARCHAR2(20)
, CONSTRAINT pk_tipus_arbitre PRIMARY KEY (id_tipus_arb));

CREATE SEQUENCE tipus_arbitre_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula EQUIP_ARBITRAL:
-------------------------------------------------------------------------------

CREATE TABLE equip_arbitral 
( partit NUMBER NOT NULL
, arbitre NUMBER NOT NULL
, tipus_arbitre NUMBER NOT NULL
, CONSTRAINT pk_equip_arbitral PRIMARY KEY (partit,arbitre)
, CONSTRAINT fk_equip_arbitral_1 FOREIGN KEY (partit)
  REFERENCES partit (id_partit)
, CONSTRAINT fk_equip_arbitral_2 FOREIGN KEY (arbitre)
  REFERENCES arbitre (id_arbitre)
, CONSTRAINT fk_equip_arbitral_3 FOREIGN KEY (tipus_arbitre)
  REFERENCES tipus_arbitre (id_tipus_arb));



-------------------------------------------------------------------------------
-- Taula TCNIC:
-------------------------------------------------------------------------------

CREATE TABLE tecnic 
( id_tecnic NUMBER NOT NULL
, nom VARCHAR2(20)
, cognoms VARCHAR2(45)
, num_federat VARCHAR2(20)
, CONSTRAINT pk_tecnic PRIMARY KEY (id_tecnic));

CREATE SEQUENCE tecnic_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula CRREC_TCNIC:
-------------------------------------------------------------------------------

CREATE TABLE carrec_tecnic 
( id_carrec_tecnic NUMBER NOT NULL
, nom VARCHAR2(45)
, CONSTRAINT pk_carrec_tecnic PRIMARY KEY (id_carrec_tecnic));

CREATE SEQUENCE carrec_tecnic_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



-------------------------------------------------------------------------------
-- Taula COS_TCNIC:
-------------------------------------------------------------------------------

CREATE TABLE cos_tecnic 
( equip NUMBER NOT NULL
, tecnic NUMBER NOT NULL
, carrec_tecnic NUMBER NOT NULL
, CONSTRAINT pk_cos_tecnic PRIMARY KEY (equip,tecnic)
, CONSTRAINT fk_cos_tecnic_1 FOREIGN KEY (equip)
  REFERENCES equip (id_equip)
, CONSTRAINT fk_cos_tecnic_2 FOREIGN KEY (tecnic)
  REFERENCES tecnic (id_tecnic)
, CONSTRAINT fk_cos_tecnic_3 FOREIGN KEY (carrec_tecnic)
  REFERENCES carrec_tecnic (id_carrec_tecnic));



-------------------------------------------------------------------------------
-- Taula LOG:
-------------------------------------------------------------------------------

CREATE TABLE log 
( id_log NUMBER NOT NULL
, data_hora TIMESTAMP(0) DEFAULT SYSTIMESTAMP
, procediment VARCHAR2(45)
, parametres_in VARCHAR2(2000)
, parametres_out VARCHAR2(2000)
, CONSTRAINT pk_log PRIMARY KEY (id_log));

CREATE SEQUENCE log_seq INCREMENT BY 1 START WITH 1 MINVALUE 1 ORDER;



