/**
 * Estudiant: David Carreras Gresely
 * TFC: Disseny i implementaci de la base de dades dun sistema 
 *      de gesti d'informaci de jugadors de bsquet.
 * Data: 12-06-2013
 * Document: Creacio_Taules_Mod_Estad.sql
 * Contingut: Aquest document cont els scripts per crear les taules del Mdul
 *            Estadstic de la BD.
 *            Lordre en que es mostren coincideix amb lordre en que shan de 
 *            crear les taules per tal de no provocar cap conflicte amb les 
 *            claus foranes.
 */


-------------------------------------------------------------------------------
-- Taula ME_TOTAL_JUGADORS_ACTIU
-------------------------------------------------------------------------------

CREATE TABLE me_total_jugadors_actiu
( id_total_jug_actiu NUMBER DEFAULT 1 NOT NULL
, total_jug_act      NUMBER DEFAULT 0
, CONSTRAINT pk_me_total_jugadors_actiu PRIMARY KEY (id_total_jug_actiu));



-------------------------------------------------------------------------------
-- Taula ME_MAX_ANOTADOR_TEMPORADA
-------------------------------------------------------------------------------

CREATE TABLE me_max_anotador_temporada
( competicio NUMBER NOT NULL 
, nom_comp   VARCHAR2(20 CHAR)
, temporada  VARCHAR2(20 CHAR)
, jugador    NUMBER DEFAULT NULL
, nom        VARCHAR2(20 CHAR)
, cognoms    VARCHAR2(45 CHAR)
, punts      NUMBER DEFAULT 0
, CONSTRAINT pk_me_max_anotador_temporada PRIMARY KEY (competicio) 
, CONSTRAINT fk_max_anotador_1 FOREIGN KEY (competicio) 
  REFERENCES competicio (id_competicio) 
, CONSTRAINT fk_max_anotador_2 FOREIGN KEY (jugador) 
  REFERENCES jugador (id_jugador));



-------------------------------------------------------------------------------
-- Taula ME_JUGADOR_MES_BEN_PAGAT
-------------------------------------------------------------------------------

CREATE TABLE me_jugador_mes_ben_pagat
( modalitat     NUMBER NOT NULL 
, genere        NUMBER NOT NULL 
, nom_modalitat VARCHAR2(20 CHAR) NOT NULL 
, nom_genere    VARCHAR2(20 CHAR) NOT NULL 
, jugador       NUMBER DEFAULT 0
, nom           VARCHAR2(20 CHAR)
, cognoms       VARCHAR2(45 CHAR)
, sou_brut      NUMBER DEFAULT 0
, CONSTRAINT pk_me_jugador_mes_ben_pagat PRIMARY KEY (modalitat, genere) 
, CONSTRAINT u_jugador_mes_pagat_1 UNIQUE (nom_modalitat) 
, CONSTRAINT u_jugador_mes_pagat_2 UNIQUE (nom_genere) 
, CONSTRAINT fk_jugador_mes_pagat_1 FOREIGN KEY (modalitat) 
  REFERENCES modalitat (id_modalitat) 
, CONSTRAINT fk_jugador_mes_pagat_2 FOREIGN KEY (genere) 
  REFERENCES genere (id_genere) 
, CONSTRAINT fk_jugador_mes_pagat_3 FOREIGN KEY (jugador) 
  REFERENCES jugador (id_jugador));



-------------------------------------------------------------------------------
-- Taula ME_JUGADOR_MES_GUANYS_ACUMULAT
-------------------------------------------------------------------------------

CREATE TABLE me_jugador_mes_guanys_acumulat
( id_jugador_mga NUMBER NOT NULL 
, jugador        NUMBER DEFAULT 0
, nom            VARCHAR2(20 CHAR)
, cognoms        VARCHAR2(45 CHAR)
, guanys         NUMBER DEFAULT 0
, CONSTRAINT pk_me_jugador_mga PRIMARY KEY (id_jugador_mga) 
, CONSTRAINT fk_me_jugador_mga_1 FOREIGN KEY (jugador) 
  REFERENCES jugador (id_jugador));



-------------------------------------------------------------------------------
-- Taula ME_SOU_MIG_ANUAL_JUGADORS
-------------------------------------------------------------------------------

CREATE TABLE me_sou_mig_anual_jugadors
( temporada     VARCHAR2(20 CHAR) NOT NULL 
, pais          VARCHAR2(20 CHAR) NOT NULL 
, modalitat     NUMBER NOT NULL 
, genere        NUMBER NOT NULL 
, nom_modalitat VARCHAR2(20 CHAR) NOT NULL 
, nom_genere    VARCHAR2(20 CHAR) NOT NULL 
, sou_mig_anual NUMBER DEFAULT 0
, CONSTRAINT pk_me_sou_mig_anual PRIMARY KEY (temporada, pais, modalitat, genere) 
, CONSTRAINT u_me_sou_mig_anual_1 UNIQUE (nom_modalitat) 
, CONSTRAINT u_me_sou_mig_anual_2 UNIQUE (nom_genere) 
, CONSTRAINT fk_me_sou_mig_anual_1 FOREIGN KEY (modalitat) 
  REFERENCES modalitat (id_modalitat) 
, CONSTRAINT fk_me_sou_mig_anual_2 FOREIGN KEY (genere) 
  REFERENCES genere (id_genere));


CREATE UNIQUE INDEX pk_me_sou_mig_anual ON me_sou_mig_anual_jugadors
  ( temporada, pais, modalitat, genere );
CREATE UNIQUE INDEX U_ME_SOU_MIG_ANUAL_1 ON me_sou_mig_anual_jugadors
  ( nom_modalitat );
CREATE UNIQUE INDEX U_ME_SOU_MIG_ANUAL_2 ON me_sou_mig_anual_jugadors
  ( nom_genere );



-------------------------------------------------------------------------------
-- Taula ME_EQUIPS_OFENSIUS_I_DEFENSIUS
-------------------------------------------------------------------------------

CREATE TABLE me_equips_ofensius_i_defensius
( competicio         NUMBER NOT NULL  
, temporada          VARCHAR2(20 CHAR) NOT NULL  
, nom_competicio     VARCHAR2(20 CHAR) NOT NULL  
, equip_ofensiu      NUMBER DEFAULT 0 
, nom_equip_ofensiu  VARCHAR2(255 CHAR) 
, punts_ofensius     NUMBER DEFAULT 0 
, equip_defensiu     NUMBER DEFAULT 0,
, nom_equip_defensiu VARCHAR2(255 CHAR) 
, punts_defensius    NUMBER DEFAULT 0 
, CONSTRAINT pk_me_equips_ofens_defens PRIMARY KEY (competicio, temporada) 
, CONSTRAINT fk_me_equips_ofens_defens_1 FOREIGN KEY (competicio) 
  REFERENCES competicio (id_competicio) 
, CONSTRAINT fk_me_equips_ofens_defens_2 FOREIGN KEY (equip_ofensiu) 
  REFERENCES equip (id_equip) 
, CONSTRAINT fk_me_equips_ofens_defens_3 FOREIGN KEY (equip_defensiu) 
  REFERENCES equip (id_equip));



-------------------------------------------------------------------------------
-- Taula ME_MILLOR_JUGADOR_DEL_MON
-------------------------------------------------------------------------------

CREATE TABLE me_millor_jugador_del_mon
( genere          NUMBER NOT NULL ,
, modalitat       NUMBER DEFAULT 0 NOT NULL 
, nom_genere      VARCHAR2(20 CHAR)
, nom_modalitat   VARCHAR2(20 CHAR)
, darrer_any      VARCHAR2(20 CHAR)
, id_jugador      NUMBER DEFAULT 0
, nom_jugador     VARCHAR2(20 CHAR)
, cognoms_jugador VARCHAR2(45 CHAR)
, mitjana_vp      NUMBER DEFAULT 0
, CONSTRAINT pk_me_millor_jugador_del_mon PRIMARY KEY (genere, modalitat) 
, CONSTRAINT fk_me_millor_jugador_del_mon_1 FOREIGN KEY (genere) 
  REFERENCES genere (id_genere) 
, CONSTRAINT fk_me_millor_jugador_del_mon_2 FOREIGN KEY (modalitat) 
  REFERENCES modalitat (id_modalitat) 
, CONSTRAINT fk_me_millor_jugador_del_mon_3 FOREIGN KEY (id_jugador) 
  REFERENCES jugador (id_jugador));
  
  
