/**
 * Estudiant: David Carreras Gresely
 * TFC: Disseny i implementaci de la base de dades dun sistema 
 *      de gesti d'informaci de jugadors de bsquet.
 * Data: 12-06-2013
 * Document: Creacio_Triggers.sql
 * Contingut: Aquest document cont els scripts per la creaci, de tots i cada un dels 
 *            set Disparadors utilitzats per proporcionar dades al Mdul Estadstic.
 */
 







/**
 * -------------------------------------------------------------------------------
 * -- Disparador: TOTAL_JUGADORS_ACTIU 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: El disparador sactiva cada cop que es fa un INSERT, o un UPDATE de la columna activitat , o be un DELETE, de una 
 *             fila de la taula jugador. Un cop activat, el disparador te dos parts. La primera sexecuta abans de fer cap canvi 
 *             en la fila afectada, i consulta el gnere i modalitat de joc del jugador afectat. Desprs, un segon bloc sexecuta 
 *             desprs que shagin fets tots els canvis en la taula, i fa el recompte del nombre de jugadors en actiu noms 
 *             daquella combinaci de gnere i modalitat. Quan obt el resultat, utilitza la instrucci MERGE per inserir, o be 
 *             actualitzar les dades en la taula del Mdul Estadstic.
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ===============================================================================================================================
 * Nom del disparador:                                                          total_jugadors_actiu
 * -------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:                         El nmero total de jugadors en actiu en tots els 
 *                                                                              gneres i modalitats.
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:                                     me_total_jugadors_actiu
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:                                     jugador
 * -------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                                           INSERT, UPDATE (COLUMNA: activitat), DELETE
 * -------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                                               Abans i desprs de canviar el contingut de la taula
 * -------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic per a cada parella gnere-modalitat:	 modalitat     (es el Id de la modalitat)
 *                                                                               genere        (es el Id del genere)
 *                                                                               nom_modalitat (es el nom de la modalitat)
 *                                                                               nom_genere    (es el nom del genere)
 *                                                                               total_jug_act (es el nombre de jugadors en actiu)
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
TRIGGER total_jugadors_actiu
  FOR INSERT OR                   -- es dispara quan es fa un INSERT,
      UPDATE OF activitat OR      -- o un UPDATE de la columna 'activitat',
      DELETE                      -- o s'esborra una fila
  ON jugador                      -- de la taula 'jugador'

  COMPOUND TRIGGER
    TYPE idgen_t IS TABLE OF genere.id_genere%TYPE;        -- defineix diversos tipus Taula que serviran
    TYPE idmod_t IS TABLE OF modalitat.id_modalitat%TYPE;
    TYPE nmgen_t IS TABLE OF genere.nom%TYPE;
    TYPE nmmod_t IS TABLE OF modalitat.nom%TYPE;
    idg idgen_t := idgen_t(0);               -- .. per guardar els id_genere de competicio
    nmg nmgen_t := nmgen_t(' ');             -- .. per guardar els noms de genere de competicio
    idm idmod_t := idmod_t(0);               -- .. per guardar els id_modalitat de competicio
    nmm nmmod_t := nmmod_t(' ');             -- .. per guardar els noms de modalitat de competicio
    seguent  NUMBER := 0;                    -- indicar la seguent posicio lliure dins de cada taula
  
-- aquest bloc sexecuta cada cop abans de canviar cada una de les files afectades.
BEFORE EACH ROW IS
  BEGIN
    seguent := seguent+1;              -- incrementa la posicio on es guardaran les dades de la nova fila
  
    IF INSERTING OR UPDATING THEN
      idg(seguent) := :NEW.genere;        -- obt el Id del genere en cas de INSERT o UPDATE
      idm(seguent) := :NEW.modalitat;     -- obt el Id de la modalitat en cas de INSERT o UPDATE
    END IF;
  
    IF DELETING THEN
      idg(seguent) := :OLD.genere;        -- obt el Id del gnere en cas de DELETE
      idm(seguent) := :OLD.modalitat;     -- obt el Id de la modalitat en cas de DELETE
    END IF;  
    SELECT nom INTO nmg(seguent) FROM genere WHERE id_genere = idg(seguent);    -- obt el nom del gnere
    SELECT nom INTO nmm(seguent) FROM modalitat WHERE id_modalitat = idm(seguent);  -- obt nom de modalitat
    idg.EXTEND(1);                        -- afegeix un element mes a cada taula
    idm.EXTEND(1);
    nmg.EXTEND(1);
    nmm.EXTEND(1);

  END BEFORE EACH ROW;
  
  -- aquest bloc sexecuta un sol cop desprs de canviar totes les possibles files.
  AFTER STATEMENT IS
  BEGIN
    FOR j IN 1..seguent LOOP			-- recorre totes les files afectades
      MERGE INTO me_total_jugadors_actiu D 
        USING (SELECT COUNT(*) AS tja       -- conta el nombre de jugadors en actiu de la modalitat i 
               FROM jugador j               -- genere de cada possible fila afectada.
               WHERE j.activitat = 'actiu' 
                 AND j.modalitat = idm(j)  
                 AND j.genere = idg(j) ) S
        ON (D.modalitat = idm(j) AND D.genere = idg(j))  -- comprova si en la taula de desti, ja existeix 
        WHEN MATCHED THEN                                -- una entrada per aquesta modalitat i genere.
          UPDATE SET D.nom_modalitat = nmm(j),           -- si hi es, actualitza les dades
                     D.nom_genere = nmg(j),
                     D.total_jug_act = S.tja 
          WHERE D.modalitat = idm(j) 
            AND D.genere = idg(j) 
        WHEN NOT MATCHED THEN                            -- si no hi es, n'entra una de nova
          INSERT (D.modalitat, D.genere, D.nom_modalitat, D.nom_genere, D.total_jug_act) 
          VALUES (idm(j), idg(j), nmm(j), nmg(j), S.tja);
    END LOOP;                                            -- acaba el bucle 

  END AFTER STATEMENT;
END;
/
 







/**
 * -------------------------------------------------------------------------------
 * -- Disparador: MAX_ANOTADOR_TEMPORADA 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: La funci daquest disparador es obtenir de la BD el jugador que mes punts ha anotat en la temporada en curs, o la 
 *             temporada anterior si estem entre temporades, per a la competici a la que pertany el partit corresponent a les 
 *             dades estadstiques de joc que es guardin, es modifiquin o sesborrin de la taula estadistica_jugador_partit. Les 
 *             dades resultants les guardar en la taula me_max_anotador_temporada del Mdul Estadstic. El disparador sactiva 
 *             cada cop que es fa un INSERT, o un UPDATE de la columna PT, o be un DELETE, de una o mes files de la taula 
 *             estadistica_jugador_partit. Un cop activat, el disparador te dos parts. La primera sexecuta abans de fer cap canvi 
 *             en la fila afectada, i consulta el Id del partit i el Id de la competici corresponent. Si els canvis en la taula 
 *             estadistica_jugador_partit  afecten a mes duna fila, aquest bloc sanir repetint per a cada fila afectada, 
 *             guardant una llista dels Id de competici afectats.  Desprs, un segon bloc sexecuta un sol cop desprs que shagin 
 *             fets tots els canvis en la taula, i recorre la llista dels Id de competici afectats, buscant per a cada una de les 
 *             competicions qui es el seu mxim anotador. Cada cop que obt el resultat, utilitza la instrucci MERGE per inserir, 
 *             o be actualitzar les dades en la taula del Mdul Estadstic. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ===============================================================================================================================
 * Nom del disparador:                                                max_anotador_temporada
 * -------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:	              Donada una competici, el seu mxim anotador en la temporada 
 *                                                                    en curs (o b la darrera temporada, si ens trobem en el 
 *                                                                    perode entre temporades)
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:	                          me_max_anotador_temporada
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:                           estadistica_jugador_partit
 * -------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                                 INSERT, UPDATE (COLUMNA: PT), DELETE
 * -------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                                     Abans i desprs de canviar el contingut de la taula
 * -------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic per a cada competici:       competicio (es el Id de la competici)
 *                                                                     nom_comp   (es el nom de la competici)
 *                                                                     temporada  (es lany o anys de la temporada)
 *                                                                     jugador    (es el Id del jugador)
 *                                                                     nom        (es el nom del jugador)
 *                                                                     cognoms    (son els cognoms del jugador)
 *                                                                     punts      (son els punts anotats pel jugador)
 * ===============================================================================================================================
 * 
 */


CREATE OR REPLACE
TRIGGER max_anotador_temporada
  FOR INSERT OR                               -- es dispara quan es fa un INSERT, 
      UPDATE OF PT OR                         -- o un UPDATE de la columna 'PT',
      DELETE                                  -- o s'esborra una fila
  ON estadistica_jugador_partit               -- de la taula 'estadistica_jugador_partit'

  COMPOUND TRIGGER
    id_part NUMBER;                 		  -- var temp id del partit
    TYPE idcomp_t IS TABLE OF competicio.id_competicio%TYPE;  -- defineix un tipus Taula que servir
    idcomp idcomp_t := idcomp_t(0);		  -- per guardar els id_competicio de les files afectades
    seguent  NUMBER := 0;			  -- indicar la seguent posicio lliure dins la taula idcomp

-- aquest bloc sexecuta cada cop abans de canviar cada una de les files afectades.
BEFORE EACH ROW IS
BEGIN
  seguent := seguent+1;            -- incrementa la posicio on es guardaran les dades de la nova fila
  
  IF INSERTING OR UPDATING THEN
   id_part := :NEW.partit;         -- obt el Id del partit en cas de INSERT o UPDATE
  END IF;
  IF DELETING THEN
    id_part := :OLD.partit;        -- obt el Id del partit en cas de DELETE
  END IF;  
  
  SELECT c.id_competicio           -- obt el id_competicio que correspon al id_partit
  INTO idcomp(seguent)             -- i el guarda en la taula temporal
  FROM competicio c, partit p 
  WHERE c.id_competicio = p.competicio 
    AND p.id_partit = id_part;

  idcomp.EXTEND(1);                               -- afegeix un element mes al final de la taula per si hi ha mes files
END BEFORE EACH ROW;

-- aquest bloc sexecuta un sol cop desprs de canviar totes les possibles files.
AFTER STATEMENT IS 
  BEGIN
    FOR j IN 1..seguent LOOP			-- recorre totes les files afectades 
      MERGE INTO me_max_anotador_temporada D
        USING (SELECT *  			-- i de la competicio de cada fila, busca el seu mxim anotador
               FROM (
                 SELECT SUM(es.PT) AS maxp, 
                        j.id_jugador, 
                        j.nom, 
                        j.cognoms, 
                        c.nom AS nom_compet, 
                        c.temporada AS temp_comp 
                 FROM jugador j, estadistica_jugador_partit es, partit p, competicio c  
                 WHERE c.id_competicio = idcomp(j)  
                   AND c.id_competicio = p.competicio 
                   AND p.id_partit = es.partit 
                   AND es.jugador = j.id_jugador 
                 GROUP BY j.id_jugador, j.nom, j.cognoms, c.nom, c.temporada 
                 ORDER BY maxp DESC)
               WHERE ROWNUM = 1 ) S
            ON (D.competicio = idcomp(j))              -- comprova si en la taula de dest, ja existeix
            WHEN MATCHED THEN                          -- una entrada per aquesta competicio
              UPDATE                                   -- si hi es, actualitza les dades
              SET D.nom_comp = S.nom_compet,
                  D.temporada = S.temp_comp,
                  D.jugador = S.id_jugador,
                  D.nom = S.nom,
                  D.cognoms = S.cognoms, 
                  D.punts = S.maxp 
              WHERE competicio = idcomp(j) 
            WHEN NOT MATCHED THEN                       -- si no ho es, n'entra una de nova
              INSERT (D.competicio, D.nom_comp, D.temporada, D.jugador, D.nom, D.cognoms, D.punts) 
              VALUES (idcomp(j), S.nom_compet, S.temp_comp, S.id_jugador, S.nom, S.cognoms, S.maxp);
    END LOOP;                                           -- acaba el bucle 
  END AFTER STATEMENT;
END;
/








/**
 * -------------------------------------------------------------------------------
 * -- Disparador: JUGADOR_MES_BEN_PAGAT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: La funci daquest disparador es obtenir de la BD el jugador mes ben pagat donada una combinaci gnere-modalitat 
 *             de joc, i guardar-ho en la taula me_jugador_mes_ben_pagat del Mdul Estadstic. El disparador sactiva cada cop que 
 *             es fa un INSERT, o un UPDATE de la columna sou_brut_anual, o be un DELETE, de una o mes files de la taula contracte. 
 *             Un cop activat, el disparador te dos parts. La primera sexecuta abans de fer cap canvi en la fila afectada, i 
 *             consulta la modalitat i gnere corresponent. Si els canvis en la taula contracte  afecten a mes duna fila, aquest 
 *             bloc sanir repetint per a cada fila afectada, guardant una llista de modalitats i gneres afectats.  Desprs, un 
 *             segon bloc sexecuta un sol cop desprs que shagin fets tots els canvis en la taula, i recorre la llista de 
 *             modalitats i gneres trobada abans, fent per cada combinaci de la llista, una llista dels jugadors ordenada segons 
 *             el sou brut anual mes alt, ordenada de mes a menys. Finalment, tria el primer jugador de la llista. Cada cop que 
 *             obt el resultat, utilitza la instrucci MERGE per inserir, o be actualitzar les dades en la taula del Mdul 
 *             Estadstic. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ================================================================================================================================
 * Nom del disparador:                                                          jugador_mes_ben_pagat
 * --------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:                         El jugador ms ben pagat de cada modalitat i gnere.
 * --------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:                                     me_jugador_mes_ben_pagat
 * --------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:                                     contracte
 * --------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                                           INSERT, UPDATE (COLUMNA: sou_brut_anual), DELETE
 * --------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                                               Abans i desprs de canviar el contingut de la taula
 * --------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic per a cada parella gnere-modalitat:   modalitat      (es el Id de la modalitat)
 *                                                                               genere         (es el Id del genere)
 *                                                                               nom_modalitat  (es el nom de la modalitat)
 *                                                                               nom_genere     (es el nom del genere)
 *                                                                               jugador        (es el Id del jugador)
 *                                                                               nom            (es el nom del jugador)
 *                                                                               cognoms        (son els cognoms del jugador)
 *                                                                               sou_brut       (es el sou brut anual del jugador)
 * ================================================================================================================================
 * 
 */


CREATE OR REPLACE
TRIGGER jugador_mes_ben_pagat
  FOR INSERT OR                               -- es dispara quan es fa un INSERT,
      UPDATE OF sou_brut_anual OR             -- o un UPDATE de la columna 'sou_brut_anual'
      DELETE                                  -- o s'esborra una fila
  ON contracte                                -- de la taula 'contracte'

  COMPOUND TRIGGER
    TYPE gen_t IS TABLE OF genere%ROWTYPE;        -- defineix diversos tipus Taula que serviran ...
    TYPE mod_t IS TABLE OF modalitat%ROWTYPE;
    k genere%ROWTYPE;           -- serveix per inicialitzar la taula gen
    h modalitat%ROWTYPE;        -- serveix per inicialitzar la taula md
    gen gen_t := gen_t(k);      -- ... per guardar els id i els noms del genere de cada fila afectada
    md mod_t := mod_t(h);       -- ... per guardar els id i els noms de la modalitat de cada fila afectada
    seguent  NUMBER := 0;       -- indicar la seguent posicio lliure dins de cada taula
    idjug NUMBER;               -- var temp amb el id_jugador
    ideqc NUMBER;               -- var temp amb el id_equip

  -- aquest bloc sexecuta cada cop abans de canviar cada una de les files afectades.
  BEFORE EACH ROW IS
  BEGIN
    seguent := seguent+1;           -- incrementa la posicio on es guardaran les dades de la nova fila
  
    IF INSERTING OR UPDATING THEN
      idjug := :NEW.jugador;        -- obt el Id del jugador en cas de INSERT o UPDATE
    END IF;
  
    IF DELETING THEN
      idjug := :OLD.jugador;        -- obt el Id del jugador en cas de DELETE
    END IF;  

    SELECT * INTO gen(seguent)      -- obt el id i el nom del genere
    FROM genere 
    WHERE id_genere = (SELECT genere FROM jugador WHERE id_jugador = idjug);
    
    SELECT * INTO md(seguent)       -- obt el id i el nom de la modalitat
    FROM modalitat 
    WHERE id_modalitat = (SELECT modalitat FROM jugador WHERE id_jugador = idjug);

    gen.EXTEND(1);                  -- afegeix un element mes a cada taula
    md.EXTEND(1);
  END BEFORE EACH ROW;
  
  -- aquest bloc sexecuta un sol cop desprs de canviar totes les possibles files.
  AFTER STATEMENT IS
  BEGIN
    FOR k IN 1..seguent LOOP			            -- recorre totes les files afectades
      MERGE INTO me_jugador_mes_ben_pagat D
        USING (SELECT *                                 -- busca el jugador mes ben pagat de la modalitat
               FROM (SELECT j.id_jugador AS idj,        -- i genere de cada possible fila afectada.
                            j.nom, 
                            j.cognoms, 
                            j.modalitat, 
                            j.genere, 
                            MAX(c.sou_brut_anual) AS sou 
                     FROM contracte c,jugador j
                     WHERE c.jugador = j.id_jugador
                       AND j.modalitat = md(k).id_modalitat          
                       AND j.genere = gen(k).id_genere             
                     GROUP BY j.id_jugador, j.nom, j.cognoms, j.modalitat, j.genere 
                     ORDER BY sou DESC)
               WHERE ROWNUM <= 1 ) S
        ON ( D.modalitat = S.modalitat                  -- comprova si en la taula de desti, ja existeix
         AND D.genere = S.genere)                       -- una entrada per aquesta modalitat i genere.
        WHEN MATCHED THEN 
          UPDATE SET D.nom_modalitat = md(k).nom,       -- si hi es, actualitza les dades
                     D.nom_genere = gen(k).nom,
                     D.jugador = S.idj,
                     D.nom = S.nom,
                     D.cognoms = S.cognoms, 
                     D.sou_brut = S.sou 
          WHERE D.modalitat = md(k).id_modalitat 
            AND D.genere = gen(k).id_genere 
        WHEN NOT MATCHED THEN                           -- si no hi es, n'entra una de nova
          INSERT (D.modalitat,D.genere,D.nom_modalitat,D.nom_genere,D.jugador,D.nom,D.cognoms,D.sou_brut) 
          VALUES (md(k).id_modalitat,gen(k).id_genere,md(k).nom,gen(k).nom,S.idj,S.nom,S.cognoms,S.sou);
    END LOOP;                                           -- acaba el bucle
  END AFTER STATEMENT;
END;
/








/**
 * -------------------------------------------------------------------------------
 * -- Disparador: JUGADOR_MES_GUANYS_ACUMULAT 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: La funci daquest disparador es obtenir de la BD el jugador que ms guanys ha acumulat al llarg de tota la seva 
 *             carrera esportiva, i guardar aquestes dades en la taula me_jugador_mes_guanys_acumulat del Mdul Estadstic. El 
 *             disparador sactiva un sol cop, desprs que shagi fet un INSERT, o un UPDATE de la columna sou_brut_anual, o be 
 *             un DELETE, de una o mes files de la taula contracte. Un cop activat, el disparador creua les taules contracte i  
 *             jugador, calculant els guanys de cada jugador al llarg dels anys en tots els seus contractes. Aleshores, fa una 
 *             llista dels jugadors ordenada segons els seus guanys, de mes a menys. I finalment tria el primer jugador de la 
 *             llista. Les dades obtingudes son inserides, o be actualitzades en la corresponent taula del Mdul estadstic 
 *             mitjanant la instrucci MERGE. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ================================================================================================================================
 * Nom del disparador:                                         jugador_mes_guanys_acumulat
 * --------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:        El jugador amb ms guanys acumulats al llarg de tota la seva carrera 
 *                                                             esportiva.
 * --------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:                    me_jugador_mes_guanys_acumulat
 * --------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:                    contracte
 * --------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                          INSERT, UPDATE (COLUMNA: sou_brut_anual), DELETE
 * --------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                              Desprs de canviar el contingut de la taula
 * --------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic:                      jugador   (es el Id del jugador)
 *                                                              nom       (es el nom del jugador)
 *                                                              cognoms   (son els cognoms del jugador)
 *                                                              guanys    (son els guanys acumulats del jugador)
 * ================================================================================================================================
 * 
 */


CREATE OR REPLACE 
TRIGGER jugador_mes_guanys_acumulats
  AFTER                                -- es dispara un sol cop desprs           
    INSERT OR                          -- de fer un INSERT,                       
    UPDATE OF sou_brut_anual OR        -- o un UPDATE de la columna 'sou_brut_anual',         
    DELETE                             -- o d'esborrar una fila                   
  ON contracte                         -- de la taula 'contracte'

BEGIN
	
  MERGE INTO me_jugador_mes_guanys_acumulat D
    USING (SELECT *                        -- busca el jugador amb mes guanys acumulats
           FROM (SELECT j.id_jugador, 
                        j.nom, 
                        j.cognoms, 
                        SUM(c.sou_brut_anual * c.durada_contracte) AS guanys 
                 FROM contracte c,jugador j
                 WHERE c.jugador = j.id_jugador
                 GROUP BY j.id_jugador, j.nom, j.cognoms
                 ORDER BY guanys DESC)
           WHERE ROWNUM <= 1 ) S
    ON (D.id_jugador_mga = 1)              -- comprova si en la taula de dest, ja existeix una entrada 
    WHEN MATCHED THEN                      -- si hi es, actualitza les dades
      UPDATE SET D.jugador = S.id_jugador,
                 D.nom = S.nom,
                 D.cognoms = S.cognoms, 
                 D.guanys = S.guanys 
    WHEN NOT MATCHED THEN                  -- si no ho es, n'entra una de nova
      INSERT (D.id_jugador_mga, D.jugador, D.nom, D.cognoms, D.guanys) 
      VALUES (1, S.id_jugador, S.nom, S.cognoms, S.guanys);

END;
/








/**
 * -------------------------------------------------------------------------------
 * -- Disparador: SOU_MIG_ANUAL_JUGADORS 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: La funci daquest disparador es obtenir de la BD el sou mig anual dels Jugadors donada una combinaci 
 *             pas-temporada-gnere-modalitat de joc concreta, i guardar-ho en la taula me_sou_mig_anual_jugadors del Mdul 
 *             Estadstic. El disparador sactiva cada cop que es fa un INSERT, o un UPDATE de la columna sou_brut_anual, o be un 
 *             DELETE, de una o mes files de la taula contracte. Un cop activat, el disparador te dos parts. La primera sexecuta 
 *             abans de fer cap canvi en la fila afectada, i consulta la temporada, pas, modalitat i gnere corresponent. Si els 
 *             canvis en la taula contracte  afecten a mes duna fila, aquest bloc sanir repetint per a cada fila afectada, 
 *             guardant una llista de temporades, pasos, modalitats i gneres afectats.  Desprs, un segon bloc sexecuta un sol 
 *             cop desprs que shagin fets tots els canvis en la taula, i recorre la llista de temporades, pasos, modalitats i 
 *             gneres trobada abans, calculant per cada combinaci de la llista, el promig del sou brut anual dels jugadors que 
 *             tenen un contracte vigent. Cada cop que obt el resultat, utilitza la instrucci MERGE per inserir, o be actualitzar 
 *             les dades en la taula del Mdul Estadstic.
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ================================================================================================================================
 * Nom del disparador:                                              sou_mig_anual_jugadors
 * --------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:             Donat un pas, una temporada, un gnere i una modalitat, el sou 
 *                                                                  mig anual dels jugadors.
 * --------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:                         me_sou_mig_anual_jugadors
 * --------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:                         contracte
 * --------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                               INSERT, UPDATE (COLUMNA: sou_brut_anual), DELETE
 * --------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                                   Abans i desprs de canviar el contingut de la taula
 * --------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic per a cada combinaci      temporada     (es lany o anys de la temporada)
 * de pas-temporada-gnere-modalitat:	                             pais          (es el pas de la competicio)
 *                                                                   modalitat     (es el Id de la modalitat)
 *                                                                   genere        (es el Id del genere)
 *                                                                   nom_modalitat (es el nom de la modalitat)
 *                                                                   nom_genere    (es el nom del genere)
 *                                                                   sou_mig_anual (es el sou mig anual dels jugadors)
 * ================================================================================================================================
 * 
 */


CREATE OR REPLACE
TRIGGER sou_mig_anual_jugadors
	FOR INSERT OR                      -- es dispara quan es fa un INSERT,
	    UPDATE OF sou_brut_anual OR    -- o un UPDATE de la columna 'sou_brut_anual',
	    DELETE                         -- o s'esborra una fila                   
	ON contracte                       -- de la taula 'contracte'

  COMPOUND TRIGGER
  TYPE gen_t IS TABLE OF genere%ROWTYPE;             -- defineix diversos tipus Taula que serviran ...
  TYPE mod_t IS TABLE OF modalitat%ROWTYPE;
  TYPE pais_t IS TABLE OF competicio.pais%TYPE;
  TYPE temp_t IS TABLE OF competicio.temporada%TYPE;
  k genere%ROWTYPE;           -- serveix per inicialitzar la taula gen 
  h modalitat%ROWTYPE;        -- serveix per inicialitzar la taula md
  gen gen_t := gen_t(k);      -- ... per guardar els id i els noms del genere de cada fila afectada
  md mod_t := mod_t(h);       -- ... per guardar els id i els noms de la modalitat de cada fila afectada
  pa pais_t := pais_t(' ');   -- ... per guardar els noms dels pasos de cada fila afectada
  tm temp_t := temp_t(' ');   -- ... per guardar els anys de la signatura del contacte de cada fila afectada
  seguent  NUMBER := 0;       -- indicar la seguent posicio lliure dins de cada taula
  idjug NUMBER;               -- var temp amb el id_jugador
  ideqc NUMBER;               -- var temp amb el id_equip
  
  -- aquest bloc sexecuta cada cop abans de canviar cada una de les files afectades.
  BEFORE EACH ROW IS
  BEGIN
    seguent := seguent+1;              -- incrementa la posicio on es guardaran les dades de la nova fila
    IF INSERTING OR UPDATING THEN
      idjug := :NEW.jugador;                     -- obt el Id del jugador en cas de INSERT o UPDATE
      ideqc := :NEW.equip_comprador;             -- obt el Id de lequip comprador en cas de INSERT o UPDATE
      tm(seguent):= TO_CHAR(:NEW.data_signatura, 'YYYY');  -- lany de la signatura en cas de INSERT o UPDATE
    END IF;
  
    IF DELETING THEN
      idjug := :OLD.jugador;                              -- obt el Id del jugador en cas de DELETE
      ideqc := :OLD.equip_comprador;                      -- obt el Id de lequip comprador en cas de DELETE
      tm(seguent):= TO_CHAR(:OLD.data_signatura, 'YYYY'); -- obt lany de la signatura en cas de DELETE
    END IF;  
    
    SELECT * INTO gen(seguent)                            -- obt el id i el nom del genere
    FROM genere 
    WHERE id_genere = (SELECT genere FROM jugador WHERE id_jugador = idjug);

    SELECT * INTO md(seguent)                             -- obt el id i el nom de la modalitat
    FROM modalitat 
    WHERE id_modalitat = (SELECT modalitat FROM jugador WHERE id_jugador = idjug);

    SELECT cm.pais INTO pa(seguent)                       -- obt el nom del pais
    FROM competicio cm, competeix cx, equip e 
    WHERE cm.id_competicio = cx.competicio 
      AND cx.equip = e.id_equip 
      AND e.id_equip = ideqc;

    gen.EXTEND(1);                          -- afegeix un element mes a cada taula
    md.EXTEND(1);
    pa.EXTEND(1);
    tm.EXTEND(1);
  END BEFORE EACH ROW;

-- aquest bloc sexecuta un sol cop desprs de canviar totes les possibles files.
AFTER STATEMENT IS 
  BEGIN
    FOR j IN 1..seguent LOOP			  -- recorre totes les files afectades
      MERGE INTO me_sou_mig_anual_jugadors D
        USING (SELECT cm.temporada,           -- conta el sou mig anual dels jugadors segons la temporada, 
                      cm.pais,                -- el pais, la modalitat, i el genere, relacionats amb la 
                      cm.modalitat,           -- fila de la taula contracte que ha canviat.
                      cm.genere, 
                      AVG(c.sou_brut_anual) AS sou_mig_anual 
               FROM contracte c, equip e, competeix cx, competicio cm
               WHERE c.equip_comprador = e.id_equip 
                 AND e.id_equip = cx.equip 
                 AND cx.competicio = cm.id_competicio  
                 AND cm.modalitat = md(j).id_modalitat
                 AND cm.genere = gen(j).id_genere
                 AND cm.pais = pa(j)
                 AND SUBSTR(cm.temporada, -4, 4) = tm(j)
                 AND TO_CHAR(c.data_signatura, 'YYYY') <= SUBSTR(cm.temporada, -4, 4) 
                 AND TO_CHAR(c.data_finalitzacio, 'YYYY') >= SUBSTR(cm.temporada, -4, 4) 
               GROUP BY cm.temporada, cm.pais, cm.modalitat, cm.genere ) S
        ON ( D.temporada = S.temporada                  -- comprova si en la taula de desti, ja existeix
         AND D.pais = S.pais                            -- una entrada per aquesta temporada, pais, 
         AND D.modalitat = S.modalitat                  -- modalitat i genere.
         AND D.genere = S.genere)
        WHEN MATCHED THEN 
          UPDATE SET D.nom_modalitat = md(j).nom,       -- si hi es, actualitza les dades
                     D.nom_genere = gen(j).nom,
                     D.sou_mig_anual = S.sou_mig_anual 
          WHERE D.temporada = S.temporada 
            AND D.pais = pa(j) 
            AND D.modalitat = md(j).id_modalitat 
            AND D.genere = gen(j).id_genere 
        WHEN NOT MATCHED THEN                           -- si no hi es, n'entra una de nova
          INSERT (D.temporada, D.pais, D.modalitat, D.genere, D.nom_modalitat, D.nom_genere, D.sou_mig_anual) 
          VALUES (S.temporada,pa(j),md(j).id_modalitat,gen(j).id_genere,md(j).nom,gen(j).nom,S.sou_mig_anual);
    END LOOP;                                           -- acaba el bucle
  END AFTER STATEMENT;
END;
/
 







/**
 * -------------------------------------------------------------------------------
 * -- Disparador: EQUIPS_OFENSIUS_I_DEFENSIUS 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: La funci daquest disparador es obtenir de la BD el millor equip ofensiu i el millor equip defensiu per a cada 
 *             competici i temporada, i guardar aquestes dades en la taula me_equips_ofensius_i_defensius del Mdul Estadstic. 
 *             El disparador sactiva un sol cop, desprs que shagi fet un INSERT, o un UPDATE de la columna pt, o be un DELETE, 
 *             de una o mes files de la taula estadistica_jugador_partit. Un cop activat, el disparador consulta la llista de 
 *             competicions de totes les temporades, i mitjanant un bucle, per a cada competici i temporada fa una consulta que 
 *             obt les dades de dos subconsultes. La primera obt les dades del equip mes ofensiu, mentre que la segona obt les 
 *             dades de lequip mes defensiu. 
 *             Per trobar lequip mes ofensiu, la primera subconsulta obt la llista de punts anotats per cada equip en tots els 
 *             partits. A continuaci, torna a buscar la llista, pro aquesta vegada creuant la segona llista amb la primera segons 
 *             el partit en com de cada parella dequips. El resultat es una nova llista amb el punts marcats (ofensius) i punts 
 *             rebuts (defensius) de cada equip en cada partit. Tot seguit, agafa aquesta llista i suma tots el punts ofensius de 
 *             cada equip, obtenint un altre nova llista dequips ordenats segons el total de punts ofensius, de mes a menys. 
 *             Finalment, tria el primer equip de la llista ( el que ha marcat mes punts, el mes ofensiu ).
 *             La segona subconsulta troba lequip mes defensiu de forma similar a com ho fa la primera subconsulta per trobar 
 *             lequip mes ofensiu. 
 *             En aquest cas pro, quan obt la llista amb el punts marcats (ofensius) i punts rebuts (defensius) de cada equip en 
 *             cada partit, el que fa es sumar tots els punts defensius de cada equip, obtenint un altre nova llista dequips 
 *             ordenats segons el total de punts defensius, de menys a mes. Finalment, tria el primer equip de la llista ( el que 
 *             ha rebut menys punts, el mes defensiu ).
 *             Un cop el disparador ha obtingut els resultats de les dos subconsultes, utilitza la instrucci MERGE per inserir, 
 *             o be actualitzar les dades en la taula del Mdul Estadstic.
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ===============================================================================================================================
 * Nom del disparador:                                    equips_ofensius_i_defensius
 * -------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:   Donada una competici i una temporada, els millors equips ofensius i 
 *                                                        defensius.
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:               me_equips_ofensius_i_defensius
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:               estadistica_jugador_partit
 * -------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                     INSERT, UPDATE (COLUMNA: pt), DELETE
 * -------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                         Desprs de canviar el contingut de la taula
 * -------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic per a cada       competicio         (es el Id de la competici)
 * parella competici-temporada:	                       nom_comp           (es el nom de la competici)
 *                                                         temporada          (es lany o anys de la temporada)
 *                                                         equip_ofensiu      (es el Id del equip ofensiu)
 *                                                         nom_equip_ofensiu  (es el nom del equip ofensiu)
 *                                                         punts_ofensius     (es el total de punts marcats per lequip ofensiu)
 *                                                         equip_defensiu     (es el Id del equip defensiu)
 *                                                         nom_equip_defensiu (es el nom del equip defensiu)
 *                                                         punts_defensius    (punts totals que li han marcat a lequip defensiu)
 * ===============================================================================================================================
 * 
 */
 
 
CREATE OR REPLACE
TRIGGER equips_ofensius_i_defensius
  AFTER                             -- es dispara un sol cop desprs              
    INSERT OR                       -- de fer un INSERT,                          
    UPDATE OF pt OR                 -- o un UPDATE de la columna 'pt',
    DELETE                          -- o d'esborrar una fila                      
  ON estadistica_jugador_partit     -- de la taula 'estadistica_jugador_partit'                    

DECLARE
  idcmp    NUMBER;                  -- var temp id_competicio
  nomcmp   VARCHAR2(20);            -- var temp nom de la competicio
  temp     VARCHAR2(20);            -- var temp nom de la temporada de la competicio
  CURSOR c_competicio IS            -- recull la llista de competicions de totes les temporades
    SELECT id_competicio, nom, temporada 
    FROM competicio;          

BEGIN
  OPEN c_competicio;                                   -- obre el cursor amb la llista de competicions
    LOOP                                               -- inicia un bucle                           
      FETCH c_competicio INTO idcmp, nomcmp, temp;     -- obtenint les dades de cada competicio      
      EXIT WHEN c_competicio%NOTFOUND;                 -- fins que no troba mes competicions          
      
      MERGE INTO me_equips_ofensius_i_defensius D
        USING (SELECT *                                -- Fa una consulta en dos subconsultes:
               FROM (SELECT *       -- primer, BUSCA EL MILLOR EQUIP OFENSIU de la competicio i temporada
                     FROM (SELECT SUM(T.punts_of) AS t_punts_of,  -- Agrupa i ordena els equips segons el
                                  T.id_equip AS equip_of,         -- total de PUNTS MARCATS, DE MES A MENYS.
                                  T.nom_equip AS nom_equip_of 
                           FROM (SELECT SUM(es.pt) AS punts_of,   -- Obt la llista de punts marcats per
                                        V.punts_def,              -- cada equip en cada partit, creuada
                                        p.id_partit,              -- amb la llista de la subconsulta mes
                                        e.id_equip,               -- interna, obtenint la llista de punts
                                        e.nom_equip               -- marcats i punts rebuts per cada equip
                                 FROM equip e,                    -- en cada partit.
                                      jugadors_convocats jc, 
                                      partit p, 
                                      competicio c, 
                                      estadistica_jugador_partit es,
                                      (SELECT SUM(es.pt) AS punts_def, p.id_partit, e.id_equip   
                                       FROM equip e, 
                                            jugadors_convocats jc,          -- Obt la llista de punts
                                            partit p,                       -- marcats per cada equip
                                            competicio c,                   -- en cada partit.
                                            estadistica_jugador_partit es 
                                       WHERE c.id_competicio = p.competicio 
                                         AND p.id_partit = es.partit 
                                         AND es.jugador = jc.jugador 
                                         AND jc.equip = e.id_equip 
                                       GROUP BY e.id_equip, p.id_partit) V
                                 WHERE c.id_competicio = p.competicio 
                                   AND p.id_partit = es.partit 
                                   AND es.jugador = jc.jugador 
                                   AND jc.equip = e.id_equip  
                                   AND p.id_partit = V.id_partit 
                                   AND e.id_equip <> V.id_equip 
                                   AND c.id_competicio = idcmp 
                                   AND c.temporada = temp
                                 GROUP BY e.id_equip, e.nom_equip, p.id_partit, V.punts_def ) T 
                           GROUP BY T.id_equip, T.nom_equip
                           ORDER BY t_punts_of DESC )
                     WHERE ROWNUM <= 1 ) ofe ,

                    (SELECT *       -- segon, BUSCA EL MILLOR EQUIP DEFENSIU de la competicio i temporada
                     FROM (SELECT SUM(T.punts_def) AS t_punts_def,  -- Agrupa i ordena els equips segons el
                                  T.id_equip AS equip_def,          -- total de PUNTS REBUTS, DE MENYS A MES.
                                  T.nom_equip AS nom_equip_def 
                           FROM (SELECT SUM(es.pt) AS punts_of,     -- Obt la llista de punts marcats per
                                        V.punts_def,                -- cada equip en cada partit, creuada
                                        p.id_partit,                -- amb la llista de la subconsulta mes
                                        e.id_equip,                 -- interna, obtenint la llista de punts
                                        e.nom_equip                 -- marcats i punts rebuts per cada equip
                                 FROM equip e, 
                                      jugadors_convocats jc,
                                      partit p,
                                      competicio c,
                                      estadistica_jugador_partit es,
                                      (SELECT SUM(es.pt) AS punts_def, p.id_partit, e.id_equip   
                                       FROM equip e, 
                                            jugadors_convocats jc,        -- Obt la llista de punts
                                            partit p,                     -- marcats per cada equip
                                            competicio c,                 -- en cada partit.
                                            estadistica_jugador_partit es 
                                       WHERE c.id_competicio = p.competicio 
                                         AND p.id_partit = es.partit 
                                         AND es.jugador = jc.jugador 
                                         AND jc.equip = e.id_equip 
                                       GROUP BY e.id_equip, p.id_partit) V
                                 WHERE c.id_competicio = p.competicio 
                                   AND p.id_partit = es.partit 
                                   AND es.jugador = jc.jugador 
                                   AND jc.equip = e.id_equip 
                                   AND p.id_partit = V.id_partit 
                                   AND e.id_equip <> V.id_equip 
                                   AND c.id_competicio = idcmp 
                                   AND c.temporada = temp
                                 GROUP BY e.id_equip, e.nom_equip, p.id_partit, V.punts_def ) T 
                           GROUP BY T.id_equip, T.nom_equip
                           ORDER BY t_punts_def ASC )
                     WHERE ROWNUM <= 1 ) def  ) S

        -- comprova si en la taula de desti, ja existeix una entrada per aquesta competicio i temporada.
        ON (D.competicio = idcmp AND D.temporada = temp)    
        WHEN MATCHED THEN                                   -- si hi es, actualitza les dades
          UPDATE SET D.nom_competicio = nomcmp, 
                     D.equip_ofensiu = S.equip_of,
                     D.nom_equip_ofensiu = S.nom_equip_of, 
                     D.punts_ofensius = S.t_punts_of, 
                     D.equip_defensiu = S.equip_def,
                     D.nom_equip_defensiu = S.nom_equip_def, 
                     D.punts_defensius = S.t_punts_def 
          WHERE D.competicio = idcmp 
            AND D.temporada = temp 
        WHEN NOT MATCHED THEN                               -- si no ho es, n'entra una de nova
          INSERT (D.competicio, 
                  D.temporada, 
                  D.nom_competicio, 
                  D.equip_ofensiu, 
                  D.nom_equip_ofensiu, 
                  D.punts_ofensius, 
                  D.equip_defensiu, 
                  D.nom_equip_defensiu, 
                  D.punts_defensius) 
          VALUES (idcmp, 
                  temp, 
                  nomcmp, 
                  S.equip_of, 
                  S.nom_equip_of, 
                  S.t_punts_of, 
                  S.equip_def, 
                  S.nom_equip_def, 
                  S.t_punts_def);
    END LOOP;                                               -- acaba el bucle de competicions
  CLOSE c_competicio;                                       -- tanca el cursor de competicions
END;
/
 







/**
 * -------------------------------------------------------------------------------
 * -- Disparador: MILLOR_JUGADOR_DEL_MON 
 * -------------------------------------------------------------------------------
 * 
 * DESCRIPCI: La funci daquest disparador es obtenir de la BD el jugador considerat com el millor del mon, per a cada una de 
 *             les combinacions gnere-modalitat de joc, a partir de la mitjana de les seves valoracions ponderades del darrer 
 *             iany,  guardar aquestes dades en la taula me_millor_jugador_del_mon del Mdul Estadstic. El disparador sactiva 
 *             un sol cop, desprs que shagi fet un INSERT, o un UPDATE de la columna valoracio_ponderada, o be un DELETE, de 
 *             una o mes files de la taula estadistica_jugador_partit. Un cop activat, el disparador consulta les llistes de 
 *             gneres i modalitats, i mitjanant dos bucle, lun dins laltre, va recorrent totes les combinacions de 
 *             gnere-modalitat, calculant per cada una delles, la mitjana de les valoracions ponderades de cada jugador en el 
 *             darrer any. Aleshores, fa una llista dels jugadors, ordenada segons la mitjana de valoracions ponderades, de mes 
 *             a menys, i tria el primer jugador de la llista. Cada cop que obt un resultat, utilitza la instrucci MERGE per 
 *             inserir, o be actualitzar les dades en la taula del Mdul Estadstic. 
 * 
 * 
 * ESPECIFICACIONS:
 * 
 * ===============================================================================================================================
 * Nom del disparador:                                     millor_jugador_del_mon
 * -------------------------------------------------------------------------------------------------------------------------------
 * Consulta que ha de permetre fer al Mdul Estadstic:    Per a cada gnere i modalitat, el millor jugador del mn en el darrer 
 *                                                         any (mitjana de valoracions ponderades ms alta).
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula del Mdul Estadstic on ho guarda:                me_millor_jugador_del_mon
 * -------------------------------------------------------------------------------------------------------------------------------
 * Taula de la BD que activa el disparador:                estadistica_jugador_partit
 * -------------------------------------------------------------------------------------------------------------------------------
 * Accions que activen el disparador:                      INSERT, UPDATE (COLUMNA: valoracio_ponderada), DELETE
 * -------------------------------------------------------------------------------------------------------------------------------
 * Quan sactiva:                                          Desprs de canviar el contingut de la taula
 * -------------------------------------------------------------------------------------------------------------------------------
 * Dades guardades en el Mdul estadstic per a cada        modalitat       (es el Id de la modalitat)
 * parella gnere-modalitat:                                genere          (es el Id del genere)
 *                                                          nom_modalitat   (es el nom de la modalitat)
 *                                                          nom_genere      (es el nom del genere)
 *                                                          darrer_any      (es lany de la valoraci del jugador)
 *                                                          id_jugador      (es el Id del jugador)
 *                                                          nom_jugador     (es el nom del jugador)
 *                                                          cognoms_jugador (son els cognoms del jugador)
 *                                                          mitjana_vp      (es la mitjana de valoracions ponderades del jugador)
 * ===============================================================================================================================
 *
 */


CREATE OR REPLACE 
TRIGGER millor_jugador_del_mon
  AFTER                                 -- es dispara un sol cop desprs              
    INSERT OR                           -- de fer un INSERT,                          
    UPDATE OF valoracio_ponderada OR    -- o un UPDATE de la columna 'valoracio_ponderada',
    DELETE                              -- o d'esborrar una fila                      
  ON estadistica_jugador_partit         -- de la taula 'estadistica_jugador_partit'                    

DECLARE
  idmd  NUMBER;                         -- var temp id_modalitat de competicio
  nommd VARCHAR2(20);                   -- var temp nom de la modalitat de competicio
  idgn  NUMBER;                         -- var temp id_genere de la competicio
  nomgn VARCHAR2(20);                   -- var temp nom del genere de la competicio
  d_any VARCHAR2(20);                   -- var temp darrer any de la temporada de la competicio
  CURSOR c_modalitat IS                 -- recull la llista de modalitats
    SELECT id_modalitat, nom 
    FROM modalitat;                         
  CURSOR c_genere IS                    -- recull la llista de generes
    SELECT id_genere, nom 
    FROM genere;           

BEGIN
  d_any := TO_CHAR(SysDate - INTERVAL '1' YEAR, 'YYYY');    -- obt la xifra del darrer any
  
  OPEN c_modalitat;                                         -- obre el cursor amb la llista de modalitats
    LOOP                                                    -- inicia un bucle                           
      FETCH c_modalitat INTO idmd, nommd;                   -- obtenint les dades de cada modalitat      
      EXIT WHEN c_modalitat%NOTFOUND;                       -- fins que no troba mes modalitats          
        
      OPEN c_genere;                     -- per a cada modalitat, obre el cursor amb la llista de generes
        LOOP                                                -- inicia un bucle                                              
          FETCH c_genere INTO idgn, nomgn;                  -- obtenint les dades de cada genere                            
          EXIT WHEN c_genere%NOTFOUND;                      -- fins que no troba mes generes                                
    
          MERGE INTO me_millor_jugador_del_mon D
            -- busca el millor jugador del mon de la modalitat i genere en el darrer any
            USING (SELECT *                                 
                   FROM (SELECT j.id_jugador, j.nom, j.cognoms, AVG(es.valoracio_ponderada) AS valoracio 
                         FROM jugador j, estadistica_jugador_partit es, partit p, competicio c 
                         WHERE j.id_jugador = es.jugador 
                           AND es.partit = p.id_partit 
                           AND p.competicio = c.id_competicio 
                           AND c.genere = idgn 
                           AND c.modalitat = idmd 
                           AND SUBSTR(c.temporada, -4, 4) = d_any 
                         GROUP BY j.id_jugador, j.nom, j.cognoms
                         ORDER BY valoracio DESC )
                   WHERE ROWNUM <= 1 ) S
            ON (D.modalitat = idmd AND D.genere = idgn)   -- comprova si en la taula de desti, ja existeix 
            WHEN MATCHED THEN                             -- una entrada per aquesta modalitat i genere.
              UPDATE SET D.darrer_any = d_any,            -- si hi es, actualitza les dades (nomes mante 
                         D.id_jugador = S.id_jugador,     -- les del darrer any)
                         D.nom_jugador = S.nom,
                         D.cognoms_jugador = S.cognoms,
                         D.mitjana_vp = S.valoracio 
              WHERE D.modalitat = idmd 
                AND D.genere = idgn 
            WHEN NOT MATCHED THEN                         -- si no hi es, n'entra una de nova
              INSERT (D.genere, 
                      D.modalitat, 
                      D.nom_genere, 
                      D.nom_modalitat, 
                      D.darrer_any, 
                      D.id_jugador, 
                      D.nom_jugador, 
                      D.cognoms_jugador, 
                      D.mitjana_vp) 
              VALUES (idgn, idmd, nomgn, nommd, d_any, S.id_jugador, S.nom, S.cognoms, S.valoracio);
  
        END LOOP;                                         -- acaba el bucle de generes    
      CLOSE c_genere;                                     -- tanca el cursor de generes   
    END LOOP;                                             -- acaba el bucle de modalitats 
  CLOSE c_modalitat;                                      -- tanca el cursor de modalitats
END;



